@@ -37,9 +37,7 @@ import yum.Errors
 import yum.logginglevels
 import yum.misc
 import yum.plugins
-import rpmUtils.arch
-import rpmUtils.miscutils
-from yum.packages import parsePackages
+from rpmUtils.arch import isMultiLibArch
 from yum import _
 from yum.rpmtrans import RPMTransaction
 import signal
@@ -99,6 +97,8 @@ class YumBaseCli(yum.YumBase, output.YumOutput):
         self.registerCommand(yumcommands.HelpCommand())
         self.registerCommand(yumcommands.ReInstallCommand())        
         self.registerCommand(yumcommands.DowngradeCommand())        
+        self.registerCommand(yumcommands.VersionCommand())
+        self.registerCommand(yumcommands.HistoryCommand())
 
     def registerCommand(self, command):
         for name in command.getNames():
@@ -205,8 +205,7 @@ class YumBaseCli(yum.YumBase, output.YumOutput):
 
         if opts.version:
             self.conf.cache = 1
-            yum_progs = ['yum', 'yum-metadata-parser', 'rpm',
-                         'yum-rhn-plugin']
+            yum_progs = self.run_with_package_names
             done = False
             def sm_ui_time(x):
                 return time.strftime("%Y-%m-%d %H:%M", time.gmtime(x))
@@ -419,10 +418,21 @@ class YumBaseCli(yum.YumBase, output.YumOutput):
                  _('Running rpm_check_debug'))
             msgs = self._run_rpm_check_debug()
             if msgs:
-                print _('ERROR with rpm_check_debug vs depsolve:')
+                rpmlib_only = True
+                for msg in msgs:
+                    if msg.startswith('rpmlib('):
+                        continue
+                    rpmlib_only = False
+                if rpmlib_only:
+                    print _("ERROR You need to update rpm to handle:")
+                else:
+                    print _('ERROR with rpm_check_debug vs depsolve:')
+
                 for msg in msgs:
                     print to_utf8(msg)
-    
+
+                if rpmlib_only:
+                    return 1, [_('RPM needs to be updated')]
                 return 1, [_('Please report this error in %s') % self.conf.bugtracker_url]
 
             self.verbose_logger.debug('rpm_check_debug time: %0.3f' % (time.time() - rcd_st))
@@ -687,12 +697,21 @@ class YumBaseCli(yum.YumBase, output.YumOutput):
 
             try:
                 self.reinstall(pattern=arg)
-            except yum.Errors.ReinstallError:
+            except yum.Errors.ReinstallRemoveError:
+                self._checkMaybeYouMeant(arg, always_output=False)
+            except yum.Errors.ReinstallInstallError, e:
+                ipkg = self.rpmdb.returnPackages(patterns=[arg])[0]
+                xmsg = ''
+                if 'from_repo' in ipkg.yumdb_info:
+                    xmsg = ipkg.yumdb_info.from_repo
+                    xmsg = _(' (from %s)') % xmsg
                 self.verbose_logger.log(yum.logginglevels.INFO_2,
-                                        _('No package %s%s%s available.'),
-                                        self.term.MODE['bold'], arg,
-                                        self.term.MODE['normal'])
-                self._maybeYouMeant(arg)
+                                        _('Installed package %s%s%s%s not available.'),
+                                        self.term.MODE['bold'], ipkg,
+                                        self.term.MODE['normal'], xmsg)
+            except yum.Errors.ReinstallError, e:
+                assert False, "Shouldn't happen, but just in case"
+                self.verbose_logger.log(yum.logginglevels.INFO_2, e)
         if len(self.tsInfo) > oldcount:
             return 2, [_('Package(s) to reinstall')]
         return 0, [_('Nothing to do')]
@@ -1045,7 +1064,7 @@ class YumBaseCli(yum.YumBase, output.YumOutput):
         installedByKey = self.rpmdb.searchNevra(name=pkg.name)
         comparable = []
         for instpo in installedByKey:
-            if rpmUtils.arch.isMultiLibArch(instpo.arch) == rpmUtils.arch.isMultiLibArch(pkg.arch):
+            if isMultiLibArch(instpo.arch) == isMultiLibArch(pkg.arch):
                 comparable.append(instpo)
             else:
                 self.verbose_logger.log(yum.logginglevels.DEBUG_3,
@@ -1152,13 +1171,8 @@ class YumOptionParser(OptionParser):
             # user writable cachedir. If that fails fall back to cache-only.
             if opts.cacheonly:
                 self.base.conf.cache = 1
-            elif self.base.conf.uid != 0:
-                cachedir = yum.misc.getCacheDir()
-                if cachedir is None:
-                    self.base.conf.cache = 1
-                else:
-                    self.base.conf.cachedir = cachedir
-                    self.base.repos.setCacheDir(cachedir)
+            elif not self.base.setCacheDir():
+                self.base.conf.cache = 1
 
             if opts.obsoletes:
                 self.base.conf.obsoletes = 1
