@@ -22,6 +22,7 @@ import time
 import logging
 import types
 import gettext
+import pwd
 import rpm
 
 import re # For YumTerm
@@ -40,6 +41,7 @@ from yum.constants import *
 from yum import logginglevels, _
 from yum.rpmtrans import RPMBaseCallback
 from yum.packageSack import packagesNewestByNameArch
+import yum.packages
 
 from yum.i18n import utf8_width, utf8_width_fill, utf8_text_fill
 
@@ -141,10 +143,6 @@ class YumTerm:
         self.BG_COLOR = self.__ansi_forced_BG_COLOR
 
     def reinit(self, term_stream=None, color='auto'):
-        if color == 'never':
-            self.__enabled = False
-            return
-
         self.__enabled = True
         if not hasattr(urlgrabber.progress, 'terminal_width_cached'):
             self.columns = 80
@@ -153,7 +151,6 @@ class YumTerm:
         if color == 'always':
             self.__forced_init()
             return
-        assert color == 'auto'
 
         # Output modes:
         self.MODE = {
@@ -188,6 +185,11 @@ class YumTerm:
             'white' : ''
             }
 
+        if color == 'never':
+            self.__enabled = False
+            return
+        assert color == 'auto'
+
         # Curses isn't available on all platforms
         try:
             import curses
@@ -861,7 +863,7 @@ class YumOutput:
                 item = self._sub_highlight(item, highlight, matchfor,
                                            ignore_case=True)
             if can_overflow:
-                print self.fmtKeyValFill(key, item)
+                print self.fmtKeyValFill(key, to_unicode(item))
             else:
                 print key % item
         print '\n\n'
@@ -902,8 +904,7 @@ class YumOutput:
     def listTransaction(self):
         """returns a string rep of the  transaction in an easy-to-read way."""
         
-        self.tsInfo.makelists()
-        out = u""
+        self.tsInfo.makelists(True, True)
         pkglist_lines = []
         data  = {'n' : {}, 'v' : {}, 'r' : {}}
         a_wid = 0 # Arch can't get "that big" ... so always use the max.
@@ -938,6 +939,8 @@ class YumOutput:
         for (action, pkglist) in [(_('Installing'), self.tsInfo.installed),
                             (_('Updating'), self.tsInfo.updated),
                             (_('Removing'), self.tsInfo.removed),
+                            (_('Reinstalling'), self.tsInfo.reinstalled),
+                            (_('Downgrading'), self.tsInfo.downgraded),
                             (_('Installing for dependencies'), self.tsInfo.depinstalled),
                             (_('Updating for dependencies'), self.tsInfo.depupdated),
                             (_('Removing for dependencies'), self.tsInfo.depremoved)]:
@@ -955,7 +958,9 @@ class YumOutput:
 
             pkglist_lines.append((action, lines))
 
-        if data['n']:
+        if not data['n']:
+            return u''
+        else:
             data    = [data['n'],    {}, data['v'], data['r'], {}]
             columns = [1,         a_wid,         1,         1,  5]
             columns = self.calcColumns(data, indent="  ", columns=columns,
@@ -963,7 +968,7 @@ class YumOutput:
             (n_wid, a_wid, v_wid, r_wid, s_wid) = columns
             assert s_wid == 5
 
-            out = u"""
+            out = [u"""
 %s
 %s
 %s
@@ -971,7 +976,7 @@ class YumOutput:
        self.fmtColumns(((_('Package'), -n_wid), (_('Arch'), -a_wid),
                         (_('Version'), -v_wid), (_('Repository'), -r_wid),
                         (_('Size'), s_wid)), u" "),
-       '=' * self.term.columns)
+       '=' * self.term.columns)]
 
         for (action, lines) in pkglist_lines:
             if lines:
@@ -989,21 +994,33 @@ class YumOutput:
                 totalmsg = totalmsg + msg
         
             if lines:
-                out = out + totalmsg
+                out.append(totalmsg)
 
         summary = _("""
 Transaction Summary
 %s
-Install  %5.5s Package(s)         
-Update   %5.5s Package(s)         
-Remove   %5.5s Package(s)         
-""") % ('=' * self.term.columns,
-        len(self.tsInfo.installed + self.tsInfo.depinstalled),
-        len(self.tsInfo.updated + self.tsInfo.depupdated),
-        len(self.tsInfo.removed + self.tsInfo.depremoved))
-        out = out + summary
+""") % ('=' * self.term.columns,)
+        out.append(summary)
+        num_in = len(self.tsInfo.installed + self.tsInfo.depinstalled)
+        num_up = len(self.tsInfo.updated + self.tsInfo.depupdated)
+        summary = _("""\
+Install   %5.5s Package(s)
+Upgrade   %5.5s Package(s)
+""") % (num_in, num_up,)
+        if num_in or num_up: # Always do this?
+            out.append(summary)
+        num_rm = len(self.tsInfo.removed + self.tsInfo.depremoved)
+        num_re = len(self.tsInfo.reinstalled)
+        num_dg = len(self.tsInfo.downgraded)
+        summary = _("""\
+Remove    %5.5s Package(s)
+Reinstall %5.5s Package(s)
+Downgrade %5.5s Package(s)
+""") % (num_rm, num_re, num_dg)
+        if num_rm or num_re or num_dg:
+            out.append(summary)
         
-        return out
+        return ''.join(out)
         
     def postTransactionOutput(self):
         out = ''
@@ -1125,7 +1142,8 @@ Remove   %5.5s Package(s)
 # to exit.
             # Where "interupt (ctrl-c) again" and "two" are highlighted.
             msg = _("""
- Current download cancelled, %sinterrupt (ctrl-c) again%s within %s%s%s seconds to exit.
+ Current download cancelled, %sinterrupt (ctrl-c) again%s within %s%s%s seconds
+to exit.
 """) % (hibeg, hiend, hibeg, delta_exit_str, hiend)
             self.verbose_logger.log(logginglevels.INFO_2, msg)
         elif now - self._last_interrupt < delta_exit_chk:
@@ -1146,6 +1164,8 @@ Remove   %5.5s Package(s)
         tl = urlgrabber.progress.TerminalLine(8)
         self.verbose_logger.log(logginglevels.INFO_2, "-" * tl.rest())
         dl_time = time.time() - download_start_timestamp
+        if dl_time <= 0: # This stops divide by zero, among other problems
+            dl_time = 0.01
         ui_size = tl.add(' | %5sB' % self.format_number(remote_size))
         ui_time = tl.add(' %9s' % self.format_time(dl_time))
         ui_end  = tl.add(' ' * 5)
@@ -1154,6 +1174,342 @@ Remove   %5.5s Package(s)
                               ui_bs, ui_size, ui_time, ui_end)
         self.verbose_logger.log(logginglevels.INFO_2, msg)
 
+    def _history_uiactions(self, hpkgs):
+        actions = set()
+        count = 0
+        for hpkg in hpkgs:
+            st = hpkg.state
+            if st == 'True-Install':
+                st = 'Install'
+            if st == 'Dep-Install': # Mask these at the higher levels
+                st = 'Install'
+            if st == 'Obsoleted': #  This is just a UI tweak, as we can't have
+                                  # just one but we need to count them all.
+                st = 'Obsoleting'
+            if st in ('Install', 'Update', 'Erase', 'Reinstall', 'Downgrade',
+                      'Obsoleting'):
+                actions.add(st)
+                count += 1
+        assert len(actions) <= 6
+        if len(actions) > 1:
+            return count, ", ".join([x[0] for x in sorted(actions)])
+
+        # So empty transactions work, although that "shouldn't" really happen
+        return count, "".join(list(actions))
+
+    def _pwd_ui_username(self, uid, limit=None):
+        # loginuid is set to -1 on init.
+        if uid is None or uid == 0xFFFFFFFF:
+            loginid = _("<unset>")
+            name = _("System") + " " + loginid
+            if limit is not None and len(name) > limit:
+                name = loginid
+            return name
+
+        try:
+            user = pwd.getpwuid(uid)
+            fullname = user.pw_gecos.split(';', 2)[0]
+            name = "%s <%s>" % (fullname, user.pw_name)
+            if limit is not None and len(name) > limit:
+                name = "%s ... <%s>" % (fullname.split()[0], user.pw_name)
+                if len(name) > limit:
+                    name = "<%s>" % user.pw_name
+            return name
+        except KeyError:
+            return str(uid)
+
+    def _history_list_transactions(self, extcmds):
+        tids = set()
+        pats = []
+        usertids = extcmds[1:]
+        printall = False
+        if usertids:
+            printall = True
+            if usertids[0] == 'all':
+                usertids.pop(0)
+        for tid in usertids:
+            try:
+                int(tid)
+                tids.add(tid)
+            except ValueError:
+                pats.append(tid)
+        if pats:
+            tids.update(self.history.search(pats))
+
+        if not tids and usertids:
+            self.logger.critical(_('Bad transaction IDs, or package(s), given'))
+            return None, None
+        return tids, printall
+
+    def historyListCmd(self, extcmds):
+        """ Shows the user a list of data about the history. """
+
+        tids, printall = self._history_list_transactions(extcmds)
+        if tids is None:
+            return 1, ['Failed history info']
+
+        fmt = "%-6s | %-22s | %-16s | %-14s | %-7s"
+        print fmt % ("ID", "Login user", "Date and time", "Action(s)","Altered")
+        print "-" * 79
+        fmt = "%6u | %-22.22s | %-16s | %-14s | %4u"
+        done = 0
+        limit = 20
+        if printall:
+            limit = None
+        for old in self.history.old(tids, limit=limit):
+            if not printall and done >= limit:
+                break
+
+            done += 1
+            name = self._pwd_ui_username(old.loginuid, 22)
+            tm = time.strftime("%Y-%m-%d %H:%M",
+                               time.localtime(old.beg_timestamp))
+            num, uiacts = self._history_uiactions(old.trans_data)
+            if old.altered_lt_rpmdb and old.altered_gt_rpmdb:
+                print fmt % (old.tid, name, tm, uiacts, num), "><"
+            elif old.return_code is None:
+                print fmt % (old.tid, name, tm, uiacts, num), "**"
+            elif old.altered_lt_rpmdb:
+                print fmt % (old.tid, name, tm, uiacts, num), " <"
+            elif old.altered_gt_rpmdb:
+                print fmt % (old.tid, name, tm, uiacts, num), "> "
+            else:
+                print fmt % (old.tid, name, tm, uiacts, num)
+        lastdbv = self.history.last()
+        if lastdbv is not None:
+            #  If this is the last transaction, is good and it doesn't
+            # match the current rpmdb ... then mark it as bad.
+            rpmdbv  = self.rpmdb.simpleVersion(main_only=True)[0]
+            if lastdbv.end_rpmdbversion != rpmdbv:
+                errstring = _('Warning: RPMDB has been altered since the last yum transaction.')
+                self.logger.warning(errstring)
+
+    def _history_get_transactions(self, extcmds):
+        if len(extcmds) < 2:
+            self.logger.critical(_('No transaction ID given'))
+            return None
+
+        tids = []
+        try:
+            int(extcmds[1])
+            tids.append(extcmds[1])
+        except ValueError:
+            self.logger.critical(_('Bad transaction ID given'))
+            return None
+
+        old = self.history.old(tids)
+        if not old:
+            self.logger.critical(_('Not found given transaction ID'))
+            return None
+        return old
+    def _history_get_transaction(self, extcmds):
+        old = self._history_get_transactions(extcmds)
+        if old is None:
+            return None
+        if len(old) > 1:
+            self.logger.critical(_('Found more than one transaction ID!'))
+        return old[0]
+
+    def historyInfoCmd(self, extcmds):
+        tids = set()
+        pats = []
+        for tid in extcmds[1:]:
+            try:
+                int(tid)
+                tids.add(tid)
+            except ValueError:
+                pats.append(tid)
+        if pats:
+            tids.update(self.history.search(pats))
+
+        if not tids and len(extcmds) < 2:
+            old = self.history.last()
+            if old is not None:
+                tids.add(old.tid)
+
+        if not tids:
+            self.logger.critical(_('No transaction ID, or package, given'))
+            return 1, ['Failed history info']
+
+        lastdbv = self.history.last()
+        if lastdbv is not None:
+            lasttid = lastdbv.tid
+            lastdbv = lastdbv.end_rpmdbversion
+
+        done = False
+        for tid in self.history.old(tids):
+            if lastdbv is not None and tid.tid == lasttid:
+                #  If this is the last transaction, is good and it doesn't
+                # match the current rpmdb ... then mark it as bad.
+                rpmdbv  = self.rpmdb.simpleVersion(main_only=True)[0]
+                if lastdbv != rpmdbv:
+                    tid.altered_gt_rpmdb = True
+            lastdbv = None
+
+            if done:
+                print "-" * 79
+            done = True
+            self._historyInfoCmd(tid, pats)
+
+    def _historyInfoCmd(self, old, pats=[]):
+        name = self._pwd_ui_username(old.loginuid)
+
+        print _("Transaction ID :"), old.tid
+        begtm = time.ctime(old.beg_timestamp)
+        print _("Begin time     :"), begtm
+        if old.beg_rpmdbversion is not None:
+            if old.altered_lt_rpmdb:
+                print _("Begin rpmdb    :"), old.beg_rpmdbversion, "**"
+            else:
+                print _("Begin rpmdb    :"), old.beg_rpmdbversion
+        if old.end_timestamp is not None:
+            endtm = time.ctime(old.end_timestamp)
+            endtms = endtm.split()
+            if begtm.startswith(endtms[0]): # Chop uninteresting prefix
+                begtms = begtm.split()
+                sofar = 0
+                for i in range(len(endtms)):
+                    if i > len(begtms):
+                        break
+                    if begtms[i] != endtms[i]:
+                        break
+                    sofar += len(begtms[i]) + 1
+                endtm = (' ' * sofar) + endtm[sofar:]
+            diff = _("(%s seconds)") % (old.end_timestamp - old.beg_timestamp)
+            print _("End time       :"), endtm, diff
+        if old.end_rpmdbversion is not None:
+            if old.altered_gt_rpmdb:
+                print _("End rpmdb      :"), old.end_rpmdbversion, "**"
+            else:
+                print _("End rpmdb      :"), old.end_rpmdbversion
+        print _("User           :"), name
+        if old.return_code is None:
+            print _("Return-Code    :"), "**", _("Aborted"), "**"
+        elif old.return_code:
+            print _("Return-Code    :"), _("Failure:"), old.return_code
+        else:
+            print _("Return-Code    :"), _("Success")
+        print _("Transaction performed with:")
+        for hpkg in old.trans_with:
+            prefix = " " * 4
+            state  = _('Installed')
+            ipkgs = self.rpmdb.searchNames([hpkg.name])
+            ipkgs.sort()
+            if not ipkgs:
+                state  = _('Erased')
+            elif hpkg.pkgtup in (ipkg.pkgtup for ipkg in ipkgs):
+                pass
+            elif ipkgs[-1] > hpkg:
+                state  = _('Updated')
+            elif ipkgs[0] < hpkg:
+                state  = _('Downgraded')
+            else: # multiple versions installed, both older and newer
+                state  = _('Weird')
+            print "%s%-12s %s" % (prefix, state, hpkg)
+        print _("Packages Altered:")
+        self.historyInfoCmdPkgsAltered(old, pats)
+        if old.output:
+            print _("Scriptlet output:")
+            num = 0
+            for line in old.output:
+                num += 1
+                print "%4d" % num, line
+        if old.errors:
+            print _("Errors:")
+            num = 0
+            for line in old.errors:
+                num += 1
+                print "%4d" % num, line
+
+    def historyInfoCmdPkgsAltered(self, old, pats=[]):
+        for hpkg in old.trans_data:
+            prefix = " " * 4
+            if not hpkg.done:
+                prefix = " ** "
+
+            highlight = 'normal'
+            if pats:
+                x,m,u = yum.packages.parsePackages([hpkg], pats)
+                if x or m:
+                    highlight = 'bold'
+            (hibeg, hiend) = self._highlight(highlight)
+
+            # To chop the name off we need nevra strings, str(pkg) gives envra
+            # so we have to do it by hand ... *sigh*.
+            if hpkg.epoch == '0':
+                cn = str(hpkg)
+            else:
+                cn = "%s-%s:%s-%s.%s" % (hpkg.name, hpkg.epoch,
+                                         hpkg.version, hpkg.release, hpkg.arch)
+
+            if False: pass
+            elif hpkg.state == 'Update':
+                ln = len(hpkg.name) + 1
+                cn = (" " * ln) + cn[ln:]
+                print "%s%s%-12s%s %s" % (prefix, hibeg, hpkg.state, hiend, cn)
+            elif hpkg.state == 'Downgraded':
+                ln = len(hpkg.name) + 1
+                cn = (" " * ln) + cn[ln:]
+                print "%s%s%-12s%s %s" % (prefix, hibeg, hpkg.state, hiend, cn)
+            elif hpkg.state == 'True-Install':
+                print "%s%s%-12s%s %s" % (prefix, hibeg, "Install", hiend,  cn)
+            else:
+                print "%s%s%-12s%s %s" % (prefix, hibeg, hpkg.state, hiend, cn)
+
+    def historySummaryCmd(self, extcmds):
+        tids, printall = self._history_list_transactions(extcmds)
+        if tids is None:
+            return 1, ['Failed history info']
+
+        fmt = "%-26s | %-19s | %-16s | %-8s"
+        print fmt % ("Login user", "Time", "Action(s)", "Altered")
+        print "-" * 79
+        fmt = "%-26.26s | %-19.19s | %-16s | %8u"
+        data = {'day' : {}, 'week' : {},
+                'fortnight' : {}, 'quarter' : {}, 'half' : {}, 
+                'year' : {}, 'all' : {}}
+        for old in self.history.old(tids):
+            name = self._pwd_ui_username(old.loginuid, 26)
+            period = 'all'
+            now = time.time()
+            if False: pass
+            elif old.beg_timestamp > (now - (24 * 60 * 60)):
+                period = 'day'
+            elif old.beg_timestamp > (now - (24 * 60 * 60 *  7)):
+                period = 'week'
+            elif old.beg_timestamp > (now - (24 * 60 * 60 * 14)):
+                period = 'fortnight'
+            elif old.beg_timestamp > (now - (24 * 60 * 60 *  7 * 13)):
+                period = 'quarter'
+            elif old.beg_timestamp > (now - (24 * 60 * 60 *  7 * 26)):
+                period = 'half'
+            elif old.beg_timestamp > (now - (24 * 60 * 60 * 365)):
+                period = 'year'
+            data[period].setdefault(name, []).append(old)
+        _period2user = {'day'       : _("Last day"),
+                        'week'      : _("Last week"),
+                        'fortnight' : _("Last 2 weeks"), # US default :p
+                        'quarter'   : _("Last 3 months"),
+                        'half'      : _("Last 6 months"),
+                        'year'      : _("Last year"),
+                        'all'       : _("Over a year ago")}
+        done = 0
+        for period in ('day', 'week', 'fortnight', 'quarter', 'half', 'year',
+                       'all'):
+            if not data[period]:
+                continue
+            for name in sorted(data[period]):
+                if not printall and done > 19:
+                    break
+                done += 1
+
+                hpkgs = []
+                for old in data[period][name]:
+                    hpkgs.extend(old.trans_data)
+                count, uiacts = self._history_uiactions(hpkgs)
+                uperiod = _period2user[period]
+                print fmt % (name, uperiod, uiacts, count)
+
 
 class DepSolveProgressCallBack:
     """provides text output callback functions for Dependency Solver callback"""
