@@ -34,6 +34,7 @@ class TransactionWrapper:
                          'pgpImportPubkey',
                          'pgpPrtPkts',
                          'Debug',
+                         'problems',
                          'setFlags',
                          'setVSFlags',
                          'setProbFilter',
@@ -106,9 +107,11 @@ class TransactionWrapper:
         return reserrors
             
         
-    def returnLeafNodes(self):
+    def returnLeafNodes(self, headers=False):
         """returns a list of package tuples (n,a,e,v,r) that are not required by
-           any other package on the system"""
+           any other package on the system
+           If headers is True then it will return a list of (header, index) tuples
+           """
         
         req = {}
         orphan = []
@@ -117,10 +120,11 @@ class TransactionWrapper:
         if mi is None: # this is REALLY unlikely but let's just say it for the moment
             return orphan    
             
+        # prebuild the req dict
         for h in mi:
-            tup = miscutils.pkgTupleFromHeader(h)    
             if not h[rpm.RPMTAG_REQUIRENAME]:
                 continue
+            tup = miscutils.pkgTupleFromHeader(h)    
             for r in h[rpm.RPMTAG_REQUIRENAME]:
                 if not req.has_key(r):
                     req[r] = set()
@@ -130,19 +134,31 @@ class TransactionWrapper:
         mi = self.dbMatch()
         if mi is None:
             return orphan
-     
+
+        def _return_all_provides(hdr):
+            """ Return all the provides, via yield. """
+            # These are done one by one, so that we get lazy loading
+            for prov in hdr[rpm.RPMTAG_PROVIDES]:
+                yield prov
+            for prov in hdr[rpm.RPMTAG_FILENAMES]:
+                yield prov
+
         for h in mi:
             preq = 0
             tup = miscutils.pkgTupleFromHeader(h)
-            for p in h[rpm.RPMTAG_PROVIDES] + h[rpm.RPMTAG_FILENAMES]:
+            for p in _return_all_provides(h):
                 if req.has_key(p):
                     # Don't count a package that provides its require
                     s = req[p]
                     if len(s) > 1 or tup not in s:
                         preq = preq + 1
-        
+                        break
+
             if preq == 0:
-                orphan.append(tup)
+                if headers:
+                    orphan.append((h, mi.instance()))
+                else:
+                    orphan.append(tup)
         
         return orphan
 
