@@ -18,11 +18,13 @@ import rpmUtils
 import rpmUtils.miscutils
 import rpmUtils.arch
 
+def _vertup_cmp(tup1, tup2):
+    return rpmUtils.miscutils.compareEVR(tup1, tup2)
 class Updates:
     """
     This class computes and keeps track of updates and obsoletes.
     initialize, add installed packages, add available packages (both as
-    unique lists of name, epoch, ver, rel, arch tuples), add an optional dict
+    unique lists of name, arch, ver, rel, epoch tuples), add an optional dict
     of obsoleting packages with obsoletes and what they obsolete ie::
         foo, i386, 0, 1.1, 1: bar >= 1.1.
     """
@@ -38,22 +40,30 @@ class Updates:
         self.obsoleting_dict = {} # obsoleting pkgtup -> [ obsoleted pkgtups ]
 
         self.exactarch = 1 # don't change archs by default
-        self.exactarchlist = ['kernel', 'kernel-smp', 'glibc', 'kernel-hugemem',
-                              'kernel-enterprise', 'kernel-bigmem', 'kernel-BOOT']
+        self.exactarchlist = set(['kernel', 'kernel-smp', 'glibc',
+                                  'kernel-hugemem',
+                                  'kernel-enterprise', 'kernel-bigmem',
+                                  'kernel-BOOT'])
                               
         self.myarch = rpmUtils.arch.canonArch # set this if you want to
                                               # test on some other arch
                                               # otherwise leave it alone
+        self._is_multilib = rpmUtils.arch.isMultiLibArch(self.myarch)
         
+        self._archlist = rpmUtils.arch.getArchList(self.myarch)
+
+        self._multilib_compat_arches = rpmUtils.arch.getMultiArchInfo(self.myarch)
+
         # make some dicts from installed and available
         self.installdict = self.makeNADict(self.installed, 1)
-        self.availdict = self.makeNADict(self.available, 1)
+        self.availdict = self.makeNADict(self.available, 0) # Done in doUpdate
 
         # holder for our updates dict
         self.updatesdict = {}
         self.updating_dict = {}
         #debug, ignore me
         self.debug = 0
+        self.obsoletes = {}
 
     def _delFromDict(self, dict_, keys, value):
         for key in keys:
@@ -96,7 +106,7 @@ class Updates:
     def debugprint(self, msg):
         if self.debug:
             print msg
-        
+
     def makeNADict(self, pkglist, Nonelists):
         """return lists of (e,v,r) tuples as value of a dict keyed on (n, a)
             optionally will return a (n, None) entry with all the a for that
@@ -139,28 +149,21 @@ class Updates:
         """returns a list of package tuples in a list (n, a, e, v, r)
            takes a package name, a list of archs, and a list of pkgs in
            (n, a, e, v, r) form."""
-        # go through list and throw out all pkgs not in archlist
-        matchlist = []
-        for (n, a, e, v, r) in pkglist:
-            if name == n:
-                if a in archlist:
-                    matchlist.append((n, a, e, v, r))
-
-        if len(matchlist) == 0:
-            return []
-            
-        # get all the evr's in a tuple list for returning the highest
-        verlist = []
-        for (n, a, e, v, r) in matchlist:
-            verlist.append((e,v,r))
-
-        (high_e, high_v, high_r) = self.returnNewest(verlist)
-            
         returnlist = []
-        for (n, a, e, v, r) in matchlist:
-            if (high_e, high_v, high_r) == (e, v, r):
-                returnlist.append((n,a,e,v,r))
-                
+        high_vertup = None
+        for pkgtup in pkglist:
+            (n, a, e, v, r) = pkgtup
+            # FIXME: returnlist used to _possibly_ contain things not in
+            #        archlist ... was that desired?
+            if name == n and a in archlist:
+                vertup = (e, v, r)
+                if (high_vertup is None or
+                    (_vertup_cmp(high_vertup, vertup) < 0)):
+                    high_vertup = vertup
+                    returnlist = []
+                if vertup == high_vertup:
+                    returnlist.append(pkgtup)
+
         return returnlist
            
     def condenseUpdates(self):
@@ -290,20 +293,10 @@ class Updates:
                         # make the new ones a list b/c while we _shouldn't_
                         # have multiple updaters, we might and well, it needs
                         # to be solved one way or the other <sigh>
-        newpkgs = []
         newpkgs = self.availdict
         
-        archlist = rpmUtils.arch.getArchList(self.myarch)
-                
+        archlist = self._archlist 
         for (n, a) in newpkgs.keys():
-            # remove stuff not in our archdict
-            # high log here
-            if a is None:
-                for (arch, e,v,r) in newpkgs[(n, a)]:
-                    if arch not in archlist:
-                        newpkgs[(n, a)].remove((arch, e,v,r))
-                continue
-                
             if a not in archlist:
                 # high log here
                 del newpkgs[(n, a)]
@@ -312,19 +305,12 @@ class Updates:
         # remove the older stuff - if we're doing an update we only want the
         # newest evrs                
         for (n, a) in newpkgs:
-            if a is None:
-                continue
-
             (new_e,new_v,new_r) = self.returnNewest(newpkgs[(n, a)])
-            for (e, v, r) in newpkgs[(n, a)]:
+            for (e, v, r) in newpkgs[(n, a)][:]:
                 if (new_e, new_v, new_r) != (e, v, r):
                     newpkgs[(n, a)].remove((e, v, r))
 
-                
         for (n, a) in newpkgs:
-            if a is None: # the None archs are only for lookups
-                continue
-           
             # simple ones - look for exact matches or older stuff
             if self.installdict.has_key((n, a)):
                 for (rpm_e, rpm_v, rpm_r) in self.installdict[(n, a)]:
@@ -338,7 +324,12 @@ class Updates:
                             try:
                                 newpkgs[(n, a)].remove((e, v, r))
                             except ValueError:
-                               pass
+                                pass
+
+        # Now we add the (n, None) entries back...
+        for na in newpkgs.keys():
+            all_arches = map(lambda x: (na[1], x[0], x[1], x[2]), newpkgs[na])
+            newpkgs.setdefault((na[0], None), []).extend(all_arches)
 
         # get rid of all the empty dict entries:
         for nakey in newpkgs.keys():
@@ -411,18 +402,20 @@ class Updates:
         # however, we do want to descend x86_64->noarch, sadly.
         
         archlists = []
-        if rpmUtils.arch.isMultiLibArch(arch=self.myarch):
+        if self._is_multilib:
             if rpmUtils.arch.multilibArches.has_key(self.myarch):
                 biarches = [self.myarch]
             else:
                 biarches = [self.myarch, rpmUtils.arch.arches[self.myarch]]
             biarches.append('noarch')
             
-            multicompat = rpmUtils.arch.getMultiArchInfo(self.myarch)[0]
+            multicompat = self._multilib_compat_arches[0]
             multiarchlist = rpmUtils.arch.getArchList(multicompat)
-            archlists = [ biarches, multiarchlist ]
+            archlists = [ set(biarches), set(multiarchlist) ]
+            # archlists = [ biarches, multiarchlist ]
         else:
-            archlists = [ archlist ]
+            archlists = [ set(archlist) ]
+            # archlists = [ archlist ]
             
         for n in complexupdate:
             for thisarchlist in archlists:
@@ -434,19 +427,18 @@ class Updates:
 
                 highestinstalledpkgs = self.returnHighestVerFromAllArchsByName(n,
                                          thisarchlist, tmplist)
+                hipdict = self.makeNADict(highestinstalledpkgs, 0)
                                          
                 
-                tmplist = []
-                for (a, e, v, r) in newpkgs[(n, None)]:
-                    tmplist.append((n, a, e, v, r))                        
-                highestavailablepkgs = self.returnHighestVerFromAllArchsByName(n,
-                                         thisarchlist, tmplist)
-
-                hapdict = self.makeNADict(highestavailablepkgs, 0)
-                hipdict = self.makeNADict(highestinstalledpkgs, 0)
-
-                # now we have the two sets of pkgs
                 if n in self.exactarchlist:
+                    tmplist = []
+                    for (a, e, v, r) in newpkgs[(n, None)]:
+                        tmplist.append((n, a, e, v, r))
+                    highestavailablepkgs = self.returnHighestVerFromAllArchsByName(n,
+                                             thisarchlist, tmplist)
+
+                    hapdict = self.makeNADict(highestavailablepkgs, 0)
+
                     for (n, a) in hipdict:
                         if hapdict.has_key((n, a)):
                             self.debugprint('processing %s.%s' % (n, a))
@@ -464,16 +456,25 @@ class Updates:
                     # this is where we have to have an arch contest if there
                     # is more than one arch updating with the highest ver
                     instarchs = []
-                    availarchs = []
                     for (n,a) in hipdict:
                         instarchs.append(a)
-                    for (n,a) in hapdict:
-                        availarchs.append(a)
                     
                     rpm_a = rpmUtils.arch.getBestArchFromList(instarchs, myarch=self.myarch)
+                    if rpm_a is None:
+                        continue
+
+                    tmplist = []
+                    for (a, e, v, r) in newpkgs[(n, None)]:
+                        tmplist.append((n, a, e, v, r))
+                    highestavailablepkgs = self.returnHighestVerFromAllArchsByName(n,
+                                             thisarchlist, tmplist)
+
+                    hapdict = self.makeNADict(highestavailablepkgs, 0)
+                    availarchs = []
+                    for (n,a) in hapdict:
+                        availarchs.append(a)
                     a = rpmUtils.arch.getBestArchFromList(availarchs, myarch=self.myarch)
-                    
-                    if rpm_a is None or a is None:
+                    if a is None:
                         continue
                         
                     (rpm_e, rpm_v, rpm_r) = hipdict[(n, rpm_a)][0] # there can be just one
@@ -527,6 +528,7 @@ class Updates:
             for newtup in self.updatesdict[oldtup]:
                 returnlist.append((newtup, oldtup))
         
+        # self.reduceListByNameArch() for double tuples
         tmplist = []
         if name:
             for ((n, a, e, v, r), oldtup) in returnlist:
@@ -557,6 +559,8 @@ class Updates:
         
         return returnlist
                 
+    # NOTE: This returns obsoleters and obsoletees, but narrows based on
+    # _obsoletees_ (unlike getObsoletesList). Look at getObsoletersTuples
     def getObsoletesTuples(self, newest=0, name=None, arch=None):
         """returns obsoletes for packages in a list of tuples of:
            (obsoleting naevr, installed naevr). You can specify name and/or
@@ -573,6 +577,7 @@ class Updates:
             for rpmtup in self.obsoletes[obstup]:
                 tmplist.append((obstup, rpmtup))
         
+        # self.reduceListByNameArch() for double tuples
         returnlist = []
         if name or arch:
             for (obstup, (n, a, e, v, r)) in tmplist:
@@ -589,8 +594,43 @@ class Updates:
 
         return returnlist
                         
+    # NOTE: This returns obsoleters and obsoletees, but narrows based on
+    # _obsoleters_ (like getObsoletesList).
+    def getObsoletersTuples(self, newest=0, name=None, arch=None):
+        """returns obsoletes for packages in a list of tuples of:
+           (obsoleting naevr, installed naevr). You can specify name and/or
+           arch of the obsoleting package to narrow the results.
+           You can also specify newest=1 to get the set of newest pkgs (name, arch)
+           sorted, that obsolete something"""
            
+        tmplist = []
+        obslist = self.obsoletes.keys()
+        if newest:
+            obslist = self._reduceListNewestByNameArch(obslist)
+
+        for obstup in obslist:
+            for rpmtup in self.obsoletes[obstup]:
+                tmplist.append((obstup, rpmtup))
+
+        # self.reduceListByNameArch() for double tuples
+        returnlist = []
+        if name or arch:
+            for ((n, a, e, v, r), insttup) in tmplist:
+                if name:
+                    if name == n:
+                        returnlist.append(((n, a, e, v, r), insttup))
+                        continue
+                if arch:
+                    if arch == a:
+                        returnlist.append(((n, a, e, v, r), insttup))
+                        continue
+        else:
+            returnlist = tmplist
+
+        return returnlist
            
+    # NOTE: This returns _obsoleters_, and narrows based on that (unlike
+    # getObsoletesTuples, but like getObsoletersTuples)
     def getObsoletesList(self, newest=0, name=None, arch=None):
         """returns obsoleting packages in a list of naevr tuples of just the
            packages that obsolete something that is installed. You can specify
