@@ -30,7 +30,7 @@ class FakeConf(object):
     def __init__(self):
         self.installonlypkgs = ['kernel']
         self.exclude = []
-        self.debuglevel = 0
+        self.debuglevel = 8
         self.obsoletes = True
         self.exactarch = False
         self.exactarchlist = []
@@ -49,6 +49,17 @@ class FakeRepo(object):
     def __init__(self, id=None,sack=None):
         self.id = id
         self.sack = sack
+        self.cost = 1000
+
+    def __cmp__(self, other):
+        """ Sort base class repos. by alphanumeric on their id, also
+            see __cmp__ in YumRepository(). """
+        if self.id > other.id:
+            return 1
+        elif self.id < other.id:
+            return -1
+        else:
+            return 0
 
 class FakePackage(packages.YumAvailablePackage):
 
@@ -68,6 +79,7 @@ class FakePackage(packages.YumAvailablePackage):
         self.epoch = epoch
         self.arch = arch
         self.pkgtup = (self.name, self.arch, self.epoch, self.version, self.release)
+        self.yumdb_info = {}
 
         self.prco['provides'].append((name, 'EQ', (epoch, version, release)))
 
@@ -156,7 +168,6 @@ class DepSolveProgressCallBack:
 class _DepsolveTestsBase(unittest.TestCase):
 
     res = {0 : 'empty', 2 : 'ok', 1 : 'err'}
-    canonArch = "x86_64"
 
     def __init__(self, methodName='runTest'):
         unittest.TestCase.__init__(self, methodName)
@@ -164,10 +175,9 @@ class _DepsolveTestsBase(unittest.TestCase):
         self.buildPkgs(self.pkgs)
 
     def setUp(self):
-        self._canonArch = arch.canonArch
-        arch.canonArch = self.canonArch
+        pass
     def tearDown(self):
-        arch.canonArch = self._canonArch
+        pass
 
     @staticmethod
     def buildPkgs(pkgs, *args):
@@ -284,6 +294,7 @@ class DepsolveTests(_DepsolveTestsBase):
     def resolveCode(self):
         solver = YumBase()
         solver.conf = FakeConf()
+        solver.arch.setup_arch('x86_64')
         solver.tsInfo = solver._tsInfo = self.tsInfo
         solver.rpmdb = self.rpmdb
         solver.pkgSack = self.xsack
@@ -291,10 +302,14 @@ class DepsolveTests(_DepsolveTestsBase):
         for po in self.rpmdb:
             po.repoid = po.repo.id = "installed"
         for po in self.xsack:
-            po.repoid = po.repo.id = "TestRepository"
+            if po.repo.id is None:
+                po.repo.id = "TestRepository"
+            po.repoid = po.repo.id
         for txmbr in self.tsInfo:
             if txmbr.ts_state in ('u', 'i'):
-                txmbr.po.repoid = txmbr.po.repo.id = "TestRepository"
+                if txmbr.po.repo.id is None:
+                    txmbr.po.repo.id = "TestRepository"
+                txmbr.po.repoid = txmbr.po.repo.id
             else:
                 txmbr.po.repoid = txmbr.po.repo.id = "installed"
 
@@ -325,6 +340,7 @@ class OperationsTests(_DepsolveTestsBase):
         requirements from.
         """
         depsolver = YumBaseCli()
+        depsolver.arch.setup_arch('x86_64')
         self.rpmdb = depsolver.rpmdb = FakeRpmDb()
         self.xsack = depsolver._pkgSack  = packageSack.PackageSack()
         self.repo = depsolver.repo = FakeRepo("installed")
@@ -338,7 +354,9 @@ class OperationsTests(_DepsolveTestsBase):
             po.repoid = po.repo.id = "installed"
             self.depsolver.rpmdb.addPackage(po)
         for po in available:
-            po.repoid = po.repo.id = "TestRepository"
+            if po.repo.id is None:
+                po.repo.id = "TestRepository"
+            po.repoid = po.repo.id
             self.depsolver._pkgSack.addPackage(po)
 
         self.depsolver.basecmd = args[0]
