@@ -28,7 +28,6 @@
 # $ dbus-send --system --print-reply --type=method_call \
 #   --dest=edu.duke.linux.yum /Updatesd edu.duke.linux.yum.CheckNow
 
-import gettext
 import os
 import sys
 import time
@@ -40,7 +39,7 @@ import gobject
 import smtplib
 import threading
 from optparse import OptionParser
-from email.MIMEText import MIMEText
+from email.mime.text import MIMEText
 
 
 
@@ -84,8 +83,8 @@ class UpdateEmitter(object):
         pass
 
     def setupFailed(self, error, translation_domain):
-       """Emitted when plugin initialization failed."""
-       pass
+        """Emitted when plugin initialization failed."""
+        pass
  
 
 class SyslogUpdateEmitter(UpdateEmitter):
