@@ -18,18 +18,26 @@
 """
 Configuration parser and default values for yum.
 """
+_use_iniparse = True
 
 import os
+import sys
 import warnings
 import rpm
 import copy
 import urlparse
-from parser import ConfigPreProcessor
-from iniparse import INIConfig
-from iniparse.compat import NoSectionError, NoOptionError, ConfigParser
-from iniparse.compat import ParsingError
+import shlex
+from parser import ConfigPreProcessor, varReplace
+try:
+    from iniparse import INIConfig
+    from iniparse.compat import NoSectionError, NoOptionError, ParsingError
+    from iniparse.compat import RawConfigParser as ConfigParser
+except ImportError:
+    _use_iniparse = False
+if not _use_iniparse:
+    from ConfigParser import NoSectionError, NoOptionError, ParsingError
+    from ConfigParser import ConfigParser
 import rpmUtils.transaction
-import rpmUtils.arch
 import Errors
 
 # Alter/patch these to change the default checking...
@@ -214,6 +222,10 @@ class UrlListOption(ListOption):
         
     def parse(self, s):
         out = []
+        s = s.replace('\n', ' ')
+        s = s.replace(',', ' ')
+        items = [ item.replace(' ', '%20') for item in shlex.split(s) ]
+        s = ' '.join(items)
         for url in super(UrlListOption, self).parse(s):
             out.append(self._urloption.parse(url))
         return out
@@ -656,9 +668,8 @@ class YumConf(StartupConf):
     skip_broken = BoolOption(False)
     #  Note that "instant" is the old behaviour, but group:primary is very
     # similar but better :).
-    mdpolicy = SelectionOption('group:primary',
-                               ('instant', 'group:all', 'group:main',
-                                'group:small', 'group:primary'))
+    mdpolicy = ListOption(['group:primary'])
+    #  ('instant', 'group:all', 'group:main', 'group:small', 'group:primary'))
     multilib_policy = SelectionOption('all',('best', 'all'))
                  # all == install any/all arches you can
                  # best == use the 'best  arch' for the system
@@ -687,6 +698,14 @@ class YumConf(StartupConf):
 
     color_search_match = Option('bold')
     
+    sslcacert = Option()
+    sslverify = BoolOption(True)
+    sslclientcert = Option()
+    sslclientkey = Option()
+
+    history_record = BoolOption(True)
+    history_record_packages = ListOption(['yum', 'rpm'])
+
     _reposlist = []
 
 class RepoConf(BaseConfig):
@@ -740,6 +759,17 @@ class RepoConf(BaseConfig):
     mdpolicy = Inherit(YumConf.mdpolicy)
     cost = IntOption(1000)
     
+    sslcacert = Inherit(YumConf.sslcacert)
+    sslverify = Inherit(YumConf.sslverify)
+    sslclientcert = Inherit(YumConf.sslclientcert)
+    sslclientkey = Inherit(YumConf.sslclientkey)
+
+    
+class VersionGroupConf(BaseConfig):
+    pkglist = ListOption()
+    run_with_packages = BoolOption(False)
+
+
 def readStartupConfig(configfile, root):
     '''
     Parse Yum's main configuration file and return a StartupConf instance.
@@ -771,9 +801,10 @@ def readStartupConfig(configfile, root):
     for path in startupconf.pluginpath:
         if not path[0] == '/':
             raise Errors.ConfigError("All plugin search paths must be absolute")
-
     # Stuff this here to avoid later re-parsing
     startupconf._parser = parser
+    # setup the release ver here
+    startupconf.releasever = _getsysver(startupconf.installroot, startupconf.distroverpkg)
 
     return startupconf
 
@@ -789,9 +820,9 @@ def readMainConfig(startupconf):
 
     # Set up substitution vars
     yumvars = _getEnvVar()
-    yumvars['basearch'] = rpmUtils.arch.getBaseArch()          # FIXME make this configurable??
-    yumvars['arch'] = rpmUtils.arch.getCanonArch()             # FIXME make this configurable??
-    yumvars['releasever'] = _getsysver(startupconf.installroot, startupconf.distroverpkg)
+    yumvars['basearch'] = startupconf.basearch
+    yumvars['arch'] = startupconf.arch
+    yumvars['releasever'] = startupconf.releasever
 
     # Read [main] section
     yumconf = YumConf()
@@ -803,6 +834,7 @@ def readMainConfig(startupconf):
         ir_path = yumconf.installroot + path
         ir_path = ir_path.replace('//', '/') # os.path.normpath won't fix this and
                                              # it annoys me
+        ir_path = varReplace(ir_path, yumvars)
         setattr(yumconf, option, ir_path)
     
     # Add in some extra attributes which aren't actually configuration values 
@@ -824,6 +856,20 @@ def readMainConfig(startupconf):
     
     return yumconf
 
+def readVersionGroupsConfig(configfile="/etc/yum/version-groups.conf"):
+    parser = ConfigParser()
+    confpp_obj = ConfigPreProcessor(configfile)
+    try:
+        parser.readfp(confpp_obj)
+    except ParsingError, e:
+        raise Errors.ConfigError("Parsing file failed: %s" % e)
+    ret = {}
+    for section in parser.sections():
+        ret[section] = VersionGroupConf()
+        ret[section].populate(parser, section)
+    return ret
+
+
 def getOption(conf, section, name, option):
     '''Convenience function to retrieve a parsed and converted value from a
     ConfigParser.
@@ -864,7 +910,17 @@ def _getsysver(installroot, distroverpkg):
     '''
     ts = rpmUtils.transaction.initReadOnlyTransaction(root=installroot)
     ts.pushVSFlags(~(rpm._RPMVSF_NOSIGNATURES|rpm._RPMVSF_NODIGESTS))
-    idx = ts.dbMatch('provides', distroverpkg)
+    try:
+        idx = ts.dbMatch('provides', distroverpkg)
+    except TypeError, e:
+        # This is code for "cannot open rpmdb"
+        # this is for pep 352 compliance on python 2.6 and above :(
+        if sys.hexversion < 0x02050000:
+            if hasattr(e,'message'):
+                raise Errors.YumBaseError("Error: " + str(e.message))
+            else:
+                raise Errors.YumBaseError("Error: " + str(e))
+        raise Errors.YumBaseError("Error: " + str(e))
     # we're going to take the first one - if there is more than one of these
     # then the user needs a beating
     if idx.count() == 0:
@@ -884,6 +940,10 @@ def writeRawRepoFile(repo,only=None):
     @param only: List of attributes to work on (None = All)
     It work by reading the repo file, changes the values there shall be changed and write it back to disk.
     """
+
+    if not _use_iniparse:
+        return
+
     ini = INIConfig(open(repo.repofile))
     # Updated the ConfigParser with the changed values    
     cfgOptions = repo.cfg.options(repo.id)
