@@ -15,7 +15,7 @@
 # Copyright 2005 Duke University 
 
 """
-Depedancy resolution module for yum.
+Dependency resolution module for yum.
 """
 
 import os.path
@@ -24,8 +24,7 @@ import logging
 
 import rpmUtils.transaction
 import rpmUtils.miscutils
-import rpmUtils.arch
-from rpmUtils.arch import archDifference, isMultiLibArch, getBestArch, canCoinstall
+from rpmUtils.arch import archDifference, canCoinstall
 import misc
 from misc import unique, version_tuple_to_string
 import rpm
@@ -37,7 +36,6 @@ import logginglevels
 import Errors
 import warnings
 warnings.simplefilter("ignore", Errors.YumFutureDeprecationWarning)
-from operator import itemgetter
 
 from yum import _
 
@@ -160,13 +158,6 @@ class Depsolve(object):
 
         self.verbose_logger.log(logginglevels.DEBUG_1, _('Searching pkgSack for dep: %s'),
             name)
-        # we need to check the name - if it doesn't match:
-        # /etc/* bin/* or /usr/lib/sendmail then we should fetch the 
-        # filelists.xml for all repos to make the searchProvides more complete.
-        if name[0] == '/':
-            if not misc.re_primary_filename(name):
-                self.doSackFilelistPopulate()
-            
         pkgs = self.pkgSack.searchProvides(name)
         
         
@@ -182,7 +173,7 @@ class Depsolve(object):
         for po in pkgs:
             self.verbose_logger.log(logginglevels.DEBUG_2,
                 _('Potential match for %s from %s'), name, po)
-            if name[0] == '/' and r_v is None:
+            if misc.re_filename(name) and r_v is None:
                 # file dep add all matches to the defSack
                 defSack.addPackage(po)
                 continue
@@ -555,18 +546,15 @@ class Depsolve(object):
                     self._last_req = pkg
                     return checkdeps, missingdep
 
-        # find out which arch of the ones we can choose from is closest
-        # to the arch of the requesting pkg
-        newest = provSack.returnNewestByNameArch()
-        if len(newest) > 1: # there's no way this can be zero
-                            
-            pkgresults = self._compare_providers(newest, requiringPo)
-            # take the first one...
-            best = pkgresults[0][0]                   
-                
-        elif len(newest) == 1:
-            best = newest[0]
-        
+        pkgs = provSack.returnPackages()
+        if len(pkgs) == 1: # Minor opt.
+            best = pkgs[0]
+        else:
+            #  Always do compare providers for multiple pkgs, it deals with
+            # newest etc. ... so no need to do NewestNameArch() ... and it
+            # stops compare_providers from being clever.
+            pkgresults = self._compare_providers(pkgs, requiringPo)
+            best = pkgresults[0][0]
         
         if self.rpmdb.contains(po=best): # is it already installed?
             missingdep = 1
@@ -595,13 +583,14 @@ class Depsolve(object):
             # FIXME: Don't we want .install() here, so obsoletes get done?
             txmbr = self.tsInfo.addInstall(best)
             txmbr.setAsDep(po=requiringPo)
+            txmbr.reason = "dep"
             self._last_req = best
 
             # if we had other packages with this name.arch that we found
             # before, they're not going to be installed anymore, so we
             # should mark them to be re-checked
             if upgraded.has_key(best.pkgtup):
-                map(lambda x: self.tsInfo.remove(x), upgraded[best.pkgtup])
+                map(self.tsInfo.remove, upgraded[best.pkgtup])
 
         checkdeps = 1
         
@@ -1046,7 +1035,7 @@ class Depsolve(object):
             # return the package which is closer or None for equal, or equally useless
             
             x_dist = archDifference(req_compare_arch, x.arch)
-            if isMultiLibArch(): # only go to the next one if we're multilib - 
+            if self.arch.multilib: # only go to the next one if we're multilib - 
                 if x_dist == 0: # can't really use best's arch anyway...
                     self.verbose_logger.log(logginglevels.DEBUG_4,
                         _("better arch in po %s") %(y))
@@ -1061,21 +1050,42 @@ class Depsolve(object):
             if y_dist == x_dist:
                 return None
             return x
+
+        #  Actual start of _compare_providers().
+
+        # Do a NameArch filtering, based on repo. __cmp__
+        unique_nevra_pkgs = {}
+        for pkg in pkgs:
+            if (pkg.pkgtup in unique_nevra_pkgs and
+                unique_nevra_pkgs[pkg.pkgtup].repo <= pkg.repo):
+                continue
+            unique_nevra_pkgs[pkg.pkgtup] = pkg
+        pkgs = unique_nevra_pkgs.values()
             
         pkgresults = {}
         ipkgresults = {}
 
         for pkg in pkgs:
             pkgresults[pkg] = 0
-            if self.rpmdb.contains(pkg.name):
+
+            rpmdbpkgs = self.rpmdb.searchNevra(name=pkg.name)
+            if rpmdbpkgs:
                 #  We only want to count things as "installed" if they are
                 # older than what we are comparing, because this then an update
                 # so we give preference. If they are newer then obsoletes/etc.
                 # could play a part ... this probably needs a better fix.
-                rpmdbpkgs = self.rpmdb.returnPackages(patterns=[pkg.name])
                 newest = sorted(rpmdbpkgs)[-1]
                 if newest.verLT(pkg):
-                    ipkgresults[pkg] = 0
+                    # give pkgs which are updates just a SLIGHT edge
+                    # we should also make sure that any pkg
+                    # we are giving an edge to is not obsoleted by
+                    # something else in the transaction. :(
+                    # there are many ways I hate this - this is but one
+                    ipkgresults[pkg] = 5
+            else:
+                # just b/c they're not installed pkgs doesn't mean they should
+                # be ignored entirely. Just not preferred
+                ipkgresults[pkg] = 0
 
         #  This is probably only for "renames". What happens is that pkgA-1 gets
         # obsoleted by pkgB but pkgB requires pkgA-2, now _if_ the pkgA txmbr
@@ -1106,6 +1116,8 @@ class Depsolve(object):
         
         # add the negative of the length of the name to the score
         
+        
+        lpos = {}
         for po in pkgs:
             for nextpo in pkgs:
                 if po == nextpo:
@@ -1115,9 +1127,9 @@ class Depsolve(object):
                 # treat it like it's obsoleted. The problem here is X-1
                 # accidentally provides FOO, so you release X-2 without the
                 # provide, but X-1 is still picked over a real provider.
-                na = (po.name,po.arch)
-                lpos = self.pkgSack.returnNewestByNameArch(naTup=na)
-                if not lpos or po != sorted(lpos)[-1]:
+                if po.name not in lpos:
+                    lpos[po.name] = self.pkgSack.returnNewestByName(po.name)[:1]
+                if not lpos[po.name] or not po.verEQ(lpos[po.name][0]):
                     pkgresults[po] -= 1024
 
                 obsoleted = False
@@ -1130,9 +1142,9 @@ class Depsolve(object):
                         _("%s obsoletes %s") % (nextpo, po))
 
                 if reqpo:
-                    arches = (reqpo.arch, getBestArch())
+                    arches = (reqpo.arch, self.arch.bestarch)
                 else:
-                    arches = (getBestArch(),)
+                    arches = (self.arch.bestarch,)
                 
                 for thisarch in arches:
                     res = _compare_arch_distance(po, nextpo, thisarch)
