@@ -15,12 +15,13 @@
 # Copyright 2005 Duke University 
 
 import gzip
-from cElementTree import iterparse
-
 try:
-    from cStringIO import StringIO
+    from xml.etree import cElementTree
 except ImportError:
-    from StringIO import StringIO
+    import cElementTree
+iterparse = cElementTree.iterparse
+
+from cStringIO import StringIO
 
 #TODO: document everything here
 
@@ -51,8 +52,7 @@ class MDParser:
             raise ValueError('Unknown repodata type "%s" in %s' % (
                 elem.tag, filename))
         # Get the total number of packages
-        total = elem.get('packages', None)
-        self.total = total is None and None or int(total)
+        self.total = int(elem.get('packages', 0))
 
     def __iter__(self):
         return self
@@ -96,7 +96,7 @@ class BaseEntry:
         
     def _prefixprops(self, elem, prefix):
         ret = {}
-        for key in elem.attrib.keys():
+        for key in elem.attrib:
             ret[prefix + '_' + self._bn(key)] = elem.attrib[key]
         return ret
 
@@ -153,14 +153,13 @@ class PrimaryEntry(BaseEntry):
                 p.update(self._prefixprops(child, 'rpm_header'))
 
             elif name == 'file':
-                type = child.get('type', 'file')
+                file_type = child.get('type', 'file')
                 path = child.text
-                self.files[path] = type
+                self.files[path] = file_type
 
     def getPrco(self, elem):
         members = []
         for child in elem:
-            name = self._bn(child.tag)
             members.append(child.attrib)
         return members
         
@@ -173,9 +172,9 @@ class FilelistsEntry(BaseEntry):
         for child in elem:
             name = self._bn(child.tag)
             if name == 'file':
-                type = child.get('type', 'file')
+                file_type = child.get('type', 'file')
                 path = child.text
-                self.files[path] = type
+                self.files[path] = file_type
         elem.clear()
                 
 class OtherEntry(BaseEntry):
@@ -201,7 +200,6 @@ def test():
     for pkg in parser:
         print '-' * 40
         print pkg
-        pass
 
     print 'read: %s packages (%s suggested)' % (parser.count, parser.total)
 
