@@ -23,6 +23,33 @@ import warnings
 import re
 import fnmatch
 import misc
+from packages import parsePackages
+
+class PackageSackVersion:
+    def __init__(self):
+        self._num = 0
+        self._chksum = misc.Checksums(['sha1'])
+
+    def __str__(self):
+        return "%u:%s" % (self._num, self._chksum.hexdigest())
+
+    def __eq__(self, other):
+        if other is None: return False
+        if type(other) in (type(''), type(u'')):
+            return str(self) == other
+        if self._num != other._num: return False
+        if self._chksum.digest() != other._chksum.digest(): return False
+        return True
+    def __ne__(self, other):
+        return not (self == other)
+
+    def update(self, pkg, csum):
+        self._num += 1
+        self._chksum.update(str(pkg))
+        if csum is not None:
+            self._chksum.update(csum[0])
+            self._chksum.update(csum[1])
+
 
 class PackageSackBase(object):
     """Base class that provides the interface for PackageSacks."""
@@ -39,6 +66,20 @@ class PackageSackBase(object):
         else:
             return iter(ret)
 
+    def __cmp__(self, other):
+        if other is None:
+            return 1
+
+        s_repos = list(self.added)
+        o_repos = list(other.added)
+        if len(s_repos) != len(o_repos):
+            return len(s_repos) - len(o_repos)
+        for (s_repo, o_repo) in zip(sorted(s_repos), sorted(o_repos)):
+            ret = cmp(s_repo, o_repo)
+            if ret:
+                return ret
+        return 0
+
     def setCompatArchs(self, compatArchs):
         raise NotImplementedError()
 
@@ -133,6 +174,54 @@ class PackageSackBase(object):
         """return list of all packages"""
         raise NotImplementedError()
 
+    def addPackageExcluder(self, repoid, excluderid, excluder, *args):
+        """ Add an "excluder" for all packages in the repo/sack. Can basically
+            do anything based on nevra, changes lots of exclude decisions from
+            "preload package; test; delPackage" into "load excluder".
+            Excluderid is used so the caller doesn't have to track
+            "have I loaded the excluder for this repo.", it's probably only
+            useful when repoid is None ... if it turns out utterly worthless
+            then it's still not a huge wart. """
+        raise NotImplementedError()
+
+    def simpleVersion(self, main_only=False, groups={}):
+        """ Return a simple version for all available packages. """
+        def _up_revs(arepos, repoid, rev, pkg, csum):
+            arevs = arepos.setdefault(repoid, {})
+            rpsv = arevs.setdefault(None, PackageSackVersion())
+            rpsv.update(pkg, csum)
+            if rev is not None:
+                rpsv = arevs.setdefault(rev, PackageSackVersion())
+                rpsv.update(pkg, csum)
+
+        main = PackageSackVersion()
+        arepos = {}
+        main_grps = {}
+        arepos_grps = {}
+        for pkg in sorted(self.returnPackages()):
+            csum = pkg.returnIdSum()
+            main.update(pkg, csum)
+
+            for group in groups:
+                if pkg.name in groups[group]:
+                    if group not in main_grps:
+                        main_grps[group] = PackageSackVersion()
+                        arepos_grps[group] = {}
+                    main_grps[group].update(pkg, csum)
+
+            if main_only:
+                continue
+
+            rev = pkg.repo.repoXML.revision
+            _up_revs(arepos, pkg.repoid, rev, pkg, csum)
+            for group in groups:
+                if pkg.name in groups[group]:
+                    _up_revs(arepos_grps[group], pkg.repoid, rev, pkg, csum)
+
+        if groups:
+            return [main, arepos, main_grps, arepos_grps]
+        return [main, arepos]
+
     def returnNewestByNameArch(self, naTup=None,
                                patterns=None, ignore_case=False):
         """return list of newest packages based on name, arch matching
@@ -219,6 +308,18 @@ class PackageSackBase(object):
         """returns a list of package objects that are not required by
            any other package in this repository"""
            
+        def _return_all_provides(po):
+            """ Return all the provides, via. yield. """
+            # These are done one by one, so that we get lazy loading
+            for prov in po.provides_names:
+                yield prov
+            for prov in po.filelist:
+                yield prov
+            for prov in po.dirlist:
+                yield prov
+            for prov in po.ghostlist:
+                yield prov
+
         # fixme - maybe cache this list?
         
         req = {}
@@ -231,15 +332,18 @@ class PackageSackBase(object):
             for r in po.requires_names:
                 if not req.has_key(r):
                     req[r] = set()
-                req[r].add(po)
+                if len(req[r]) > 1: #  We only need to know if another pkg.
+                    continue        # reqs. the provide. So 2 pkgs. is enough.
+                req[r].add(po.name)
      
         for po in self.returnPackages(repoid=repoid):
             preq = 0
-            for p in po.provides_names + po.filelist + po.dirlist + po.ghostlist:
+            for p in _return_all_provides(po):
                 if req.has_key(p):
-                    # Don't count a package that provides its require
-                    s = req[p]
-                    if len(s) > 1 or po not in s:
+                    #  If this pkg provides something that is required by
+                    # anything but itself (or another version of itself) it
+                    # isn't an orphan.
+                    if len(req[p]) > 1 or po.name not in req[p]:
                         preq += 1
                         break
         
@@ -259,7 +363,7 @@ class MetaSack(PackageSackBase):
 
     def __len__(self):
         ret = 0
-        for sack in self.sacks.values():
+        for sack in sorted(self.sacks.values()):
             ret += len(sack)
         return ret
 
@@ -332,9 +436,12 @@ class MetaSack(PackageSackBase):
         
         # go through each of the keys of the obs dict and see if it is in the
         # sack of newest pkgs - if it is not - remove the entry
-        for obstup in obsdict.keys():
+        togo = []
+        for obstup in obsdict:
             if obstup not in newest_tups:
-                del obsdict[obstup]
+                togo.append(obstup)
+        for obstup in togo:
+            del obsdict[obstup]
         
         return obsdict
         
@@ -360,13 +467,29 @@ class MetaSack(PackageSackBase):
 
 
     def returnPackages(self, repoid=None, patterns=None, ignore_case=False):
-        """return list of all packages, takes optional repoid"""
+        """Returns a list of packages. Note that the packages are
+           always filtered to those matching the patterns/case. An optional
+           repoid allows you to easily get data for a specific repo. """
         if not repoid:
             return self._computeAggregateListResult("returnPackages",
                                                     None, patterns, ignore_case)
         return self.sacks[repoid].returnPackages(patterns=patterns,
                                                  ignore_case=ignore_case)
 
+    def addPackageExcluder(self, repoid, excluderid, excluder, *args):
+        """ Add an "excluder" for all packages in the repo/sack. Can basically
+            do anything based on nevra, changes lots of exclude decisions from
+            "preload package; test; delPackage" into "load excluder".
+            Excluderid is used so the caller doesn't have to track
+            "have I loaded the excluder for this repo.", it's probably only
+            useful when repoid is None ... if it turns out utterly worthless
+            then it's still not a huge wart. """
+        if not repoid:
+            calr = self._computeAggregateListResult
+            return calr("addPackageExcluder", None, excluderid, excluder, *args)
+        return self.sacks[repoid].addPackageExcluder(None,
+                                                     excluderid,excluder, *args)
+
     def returnNewestByNameArch(self, naTup=None,
                                patterns=None, ignore_case=False):
         """return list of newest packages based on name, arch matching
@@ -445,7 +568,7 @@ class MetaSack(PackageSackBase):
 
     def _computeAggregateListResult(self, methodName, *args):
         result = []
-        for sack in self.sacks.values():
+        for sack in sorted(self.sacks.values()):
             if hasattr(sack, methodName):
                 method = getattr(sack, methodName)
                 try:
@@ -460,7 +583,7 @@ class MetaSack(PackageSackBase):
 
     def _computeAggregateDictResult(self, methodName, *args):
         result = {}
-        for sack in self.sacks.values():
+        for sack in sorted(self.sacks.values()):
             if hasattr(sack, methodName):
                 method = getattr(sack, methodName)
                 try:
@@ -579,7 +702,7 @@ class PackageSack(PackageSackBase):
         self._checkIndexes(failure='build')        
         returnList = []
         if name[0] == '/':
-             returnList.extend(self.searchFiles(name))
+            returnList.extend(self.searchFiles(name))
         if self.provides.has_key(name):
             returnList.extend(self.provides[name])
         return returnList
@@ -606,11 +729,7 @@ class PackageSack(PackageSackBase):
         for po in self.returnPackages():
             if len(po.obsoletes) == 0:
                 continue
-
-            if not obs.has_key(po.pkgtup):
-                obs[po.pkgtup] = po.obsoletes
-            else:
-                obs[po.pkgtup].extend(po.obsoletes)
+            obs.setdefault(po.pkgtup, []).extend(po.obsoletes)
 
         if not newest:
             return obs
@@ -621,9 +740,12 @@ class PackageSack(PackageSackBase):
 
         # go through each of the keys of the obs dict and see if it is in the
         # sack of newest pkgs - if it is not - remove the entry
-        for obstup in obs.keys():
-            if obstup not in  newest_tups:
-                del obs[obstup]
+        togo = []
+        for obstup in obs:
+            if obstup not in newest_tups:
+                togo.append(obstup)
+        for obstup in togo:
+            del obs[obstup]
             
         return obs
         
@@ -662,10 +784,7 @@ class PackageSack(PackageSackBase):
         repoid = obj.repoid
         (name, arch, epoch, ver, rel) = obj.pkgtup
         
-        if self.compatarchs:
-            if self.compatarchs.has_key(arch):
-                self._addToDictAsList(self.pkgsByRepo, repoid, obj)
-        else:
+        if not self.compatarchs or arch in self.compatarchs:
             self._addToDictAsList(self.pkgsByRepo, repoid, obj)
         if self.indexesBuilt:
             self._addPackageToIndex(obj)
@@ -750,6 +869,10 @@ class PackageSack(PackageSackBase):
                 # nothing to return
                 pass
         
+        if patterns:
+            returnList = parsePackages(returnList, patterns, not ignore_case,
+                                       unique='repo-pkgkey')
+            returnList = returnList[0] + returnList[1]
         return returnList
 
     def returnNewestByNameArch(self, naTup=None,
