@@ -27,8 +27,8 @@ import re
 import fnmatch
 import stat
 import warnings
+from subprocess import Popen, PIPE
 from rpmUtils import RpmUtilsError
-import rpmUtils.arch
 import rpmUtils.miscutils
 from rpmUtils.miscutils import flagToString, stringToVersion
 import Errors
@@ -617,7 +617,10 @@ class YumAvailablePackage(PackageObject, RpmBase):
         # ignoring "bad" chars.
         val = _nf2ascii(val)
         # Hacky way to get rid of version numbers...
-        self._committer_ret = re.sub("""> .*""", '>', val)
+        ix = val.find('> ')
+        if ix != -1:
+            val = val[0:ix+1]
+        self._committer_ret = val
         return self._committer_ret
 
     committer  = property(_committer)
@@ -907,13 +910,13 @@ class YumAvailablePackage(PackageObject, RpmBase):
         for (name, flags, (e,v,r)) in mylist:
             pcostring = '''      <rpm:entry name="%s"''' % misc.to_xml(name, attrib=True)
             if flags:
-                pcostring += ''' flags="%s"''' % flags
+                pcostring += ''' flags="%s"''' % misc.to_xml(flags, attrib=True)
                 if e:
-                    pcostring += ''' epoch="%s"''' % e
+                    pcostring += ''' epoch="%s"''' % misc.to_xml(e, attrib=True)
                 if v:
-                    pcostring += ''' ver="%s"''' % v
+                    pcostring += ''' ver="%s"''' % misc.to_xml(v, attrib=True)
                 if r:
-                    pcostring += ''' rel="%s"''' % r
+                    pcostring += ''' rel="%s"''' % misc.to_xml(r, attrib=True)
                     
             pcostring += "/>\n"
             msg += pcostring
@@ -978,13 +981,13 @@ class YumAvailablePackage(PackageObject, RpmBase):
                 continue
             prcostring = '''      <rpm:entry name="%s"''' % misc.to_xml(name, attrib=True)
             if flags:
-                prcostring += ''' flags="%s"''' % flags
+                prcostring += ''' flags="%s"''' % misc.to_xml(flags, attrib=True)
                 if e:
-                    prcostring += ''' epoch="%s"''' % e
+                    prcostring += ''' epoch="%s"''' % misc.to_xml(e, attrib=True)
                 if v:
-                    prcostring += ''' ver="%s"''' % v
+                    prcostring += ''' ver="%s"''' % misc.to_xml(v, attrib=True)
                 if r:
-                    prcostring += ''' rel="%s"''' % r
+                    prcostring += ''' rel="%s"''' % misc.to_xml(r, attrib=True)
             if pre:
                 prcostring += ''' pre="%s"''' % pre
                     
@@ -1121,9 +1124,15 @@ class YumHeaderPackage(YumAvailablePackage):
         #FIXME - if an error - return AttributeError, not KeyError 
         # ONLY FIX THIS AFTER THE API BREAK
         if thing.startswith('__') and thing.endswith('__'):
-            if not hasattr(self, thing):
-                raise AttributeError, "%s has no attribute %s" % (self, thing)
-        return self.hdr[thing]
+            # If these existed, then we wouldn't get here ...
+            # So these are missing.
+            raise AttributeError, "%s has no attribute %s" % (self, thing)
+        try:
+            return self.hdr[thing]
+        except KeyError:
+            #  Note above, API break to fix this ... this at least is a nicer
+            # msg. so we know what we accessed that is bad.
+            raise KeyError, "%s has no attribute %s" % (self, thing)
 
     def doepoch(self):
         tmpepoch = self.hdr['epoch']
@@ -1154,24 +1163,13 @@ class YumHeaderPackage(YumAvailablePackage):
                 if not self.__mode_cache.has_key(mode):
                     self.__mode_cache[mode] = stat.S_ISDIR(mode)
           
+                fkey = 'file'
                 if self.__mode_cache[mode]:
-                    if not self.files.has_key('dir'):
-                        self.files['dir'] = []
-                    self.files['dir'].append(fn)
-                else:
-                    if flag is None:
-                        if not self.files.has_key('file'):
-                            self.files['file'] = []
-                        self.files['file'].append(fn)
-                    else:
-                        if (flag & 64):
-                            if not self.files.has_key('ghost'):
-                                self.files['ghost'] = []
-                            self.files['ghost'].append(fn)
-                            continue
-                        if not self.files.has_key('file'):
-                            self.files['file'] = []
-                        self.files['file'].append(fn)
+                    fkey = 'dir'
+                elif flag is not None and (flag & 64):
+                    fkey = 'ghost'
+                self.files.setdefault(fkey, []).append(fn)
+
             self._loadedfiles = True
             
     def returnFileEntries(self, ftype='file'):
@@ -1382,12 +1380,18 @@ class YumInstalledPackage(YumHeaderPackage):
 
             # do check of file status on system
             problems = []
-            if os.path.exists(fn):
+            if os.path.lexists(fn):
                 # stat
                 my_st = os.lstat(fn)
                 my_st_size = my_st.st_size
-                my_user  = pwd.getpwuid(my_st[stat.ST_UID])[0]
-                my_group = grp.getgrgid(my_st[stat.ST_GID])[0]
+                try:
+                    my_user  = pwd.getpwuid(my_st[stat.ST_UID])[0]
+                except KeyError, e:
+                    my_user = 'uid %s not found' % my_st[stat.ST_UID]
+                try:
+                    my_group = grp.getgrgid(my_st[stat.ST_GID])[0]
+                except KeyError, e:
+                    my_group = 'gid %s not found' % my_st[stat.ST_GID]
 
                 if mode < 0:
                     # Stupid rpm, should be unsigned value but is signed ...
@@ -1481,7 +1485,10 @@ class YumInstalledPackage(YumHeaderPackage):
                     if gen_csum and my_csum != csum and have_prelink:
                         #  This is how rpm -V works, try and if that fails try
                         # again with prelink.
-                        (ig, fp,er) = os.popen3([prelink_cmd, "-y", fn])
+                        p = Popen([prelink_cmd, "-y", fn], 
+                            shell=True, bufsize=-1, stdin=PIPE, 
+                            stdout=PIPE, stderr=PIPE, close_fds=True)
+                        (ig, fp, er) = (p.stdin, p.stdout, p.stderr)
                         # er.read(1024 * 1024) # Try and get most of the stderr
                         fp = _CountedReadFile(fp)
                         my_csum = misc.checksum(csum_type, fp)
@@ -1548,9 +1555,9 @@ class YumLocalPackage(YumHeaderPackage):
         
         try:
             hdr = rpmUtils.miscutils.hdrFromPackage(ts, self.localpath)
-        except RpmUtilsError:
+        except RpmUtilsError, e:
             raise Errors.MiscError, \
-                'Could not open local rpm file: %s' % self.localpath
+                'Could not open local rpm file: %s: %s' % (self.localpath, e)
         
         fakerepo = FakeRepository(filename)
         fakerepo.cost = 0
