@@ -13,7 +13,26 @@
 ##You should have received a copy of the GNU General Public License
 ##along with this program; if not, write to the Free Software
 ##Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-import struct, time, cStringIO, base64, types, md5, sha
+import struct, time, cStringIO, base64, types
+
+#  We use this so that we can work on python-2.4 and python-2.6, and thus.
+# use import md5/import sha on the older one and import hashlib on the newer.
+#  Stupid deprecation warnings.
+try:
+    import hashlib
+except ImportError:
+    # Python-2.4.z ... gah!
+    import sha
+    import md5
+    class hashlib:
+
+        @staticmethod
+        def new(algo):
+            if algo == 'md5':
+                return md5.new()
+            if algo == 'sha1':
+                return sha.new()
+            raise ValueError, "Bad checksum type"
 
 debug = None
 
@@ -378,14 +397,14 @@ class public_key(pgp_packet) :
         # otherwise calculate it now and cache it
         # v3 and v4 are calculated differently
         if self.version == 3 :
-            h = md5.new()
+            h = hashlib.new('md5')
             h.update(pack_long(self.pk_rsa_mod))
             h.update(pack_long(self.pk_rsa_exp))
             self.fingerprint_ = h.digest()
         elif self.version == 4 :
             # we hash what would be the whole PGP message containing
             # the pgp certificate
-            h = sha.new()
+            h = hashlib.new('sha1')
             h.update('\x99')
             # we need to has the length of the packet as well
             buf = self.serialize()
@@ -430,7 +449,7 @@ class public_key(pgp_packet) :
         idx_save = idx
         self.version, idx = get_whole_int(msg, idx, 1)
         if self.version != 2 and self.version != 3 and self.version != 4 :
-            raise 'unknown public key packet version %d at %d' % (self.version, idx_save)
+            raise RuntimeError('unknown public key packet version %d at %d' % (self.version, idx_save))
         if self.version == 2 : # map v2 into v3 for coding simplicity since they're structurally the same
             self.version = 3
         self.timestamp, idx = get_whole_number(msg, idx, 4)
@@ -452,7 +471,7 @@ class public_key(pgp_packet) :
             self.pk_elgamal_grp_gen_g, idx = get_mpi(msg, idx)
             self.pk_elgamal_pub_key, idx = get_mpi(msg, idx)
         else :
-            raise "unknown public key algorithm %d at %d" % (self.pk_algo, idx_save)
+            raise RuntimeError("unknown public key algorithm %d at %d" % (self.pk_algo, idx_save))
 
     def __str__(self) :
         sio = cStringIO.StringIO()
@@ -549,7 +568,7 @@ class signature(pgp_packet) :
             idx = idx + sublen - 1
             return (subtype, expr), idx
         if subtype == SIG_SUB_TYPE_PREF_SYMM_ALGO or subtype == SIG_SUB_TYPE_PREF_HASH_ALGO or subtype == SIG_SUB_TYPE_PREF_COMP_ALGO or subtype == SIG_SUB_TYPE_KEY_FLAGS :
-            algo_list = map(lambda x : ord(x), list(msg[idx:idx+sublen-1]))
+            algo_list = map(ord, list(msg[idx:idx+sublen-1]))
             idx = idx + sublen - 1
             return (subtype, algo_list), idx
         if subtype == SIG_SUB_TYPE_REVOKE_KEY : # revocation key
@@ -727,7 +746,7 @@ class signature(pgp_packet) :
                 sp, idx = self.deserialize_subpacket(msg, idx)
                 self.unhashed_subpaks.append(sp)
         else :
-            raise 'unknown signature packet version %d at %d' % (self.version, idx)
+            raise RuntimeError('unknown signature packet version %d at %d' % (self.version, idx))
         self.hash_frag, idx = get_whole_number(msg, idx, 2)
         if self.pk_algo == ALGO_PK_RSA_ENC_OR_SIGN or self.pk_algo == ALGO_PK_RSA_SIGN_ONLY :
             self.rsa_sig, idx = get_mpi(msg, idx)
@@ -735,7 +754,7 @@ class signature(pgp_packet) :
             self.dsa_sig_r, idx = get_mpi(msg, idx)
             self.dsa_sig_s, idx = get_mpi(msg, idx)
         else :
-            raise 'unknown public-key algorithm (%d) in signature at %d' % (self.pk_algo, idx)
+            raise RuntimeError('unknown public-key algorithm (%d) in signature at %d' % (self.pk_algo, idx))
         return idx
 
     def __str__(self) :
@@ -1060,7 +1079,7 @@ def decode(msg) :
             if debug :
                 debug.write(pkt.__str__() + "\n")
         else :
-            raise 'unknown pgp packet type %d at %d' % (pkt_typ, idx)
+            raise RuntimeError('unknown pgp packet type %d at %d' % (pkt_typ, idx))
 
         pkt_list.append(pkt)
 
