@@ -25,7 +25,7 @@ from rpmUtils.transaction import initReadOnlyTransaction
 import misc
 import Errors
 from packages import YumInstalledPackage, parsePackages
-from packageSack import PackageSackBase
+from packageSack import PackageSackBase, PackageSackVersion
 
 # For returnPackages(patterns=)
 import fnmatch
@@ -65,9 +65,10 @@ class RPMInstalledPackage(YumInstalledPackage):
     def __getattr__(self, varname):
         self.hdr = val = self._get_hdr()
         self._has_hdr = True
+        # If these existed, then we wouldn't get here ... and nothing in the DB
+        # starts and ends with __'s. So these are missing.
         if varname.startswith('__') and varname.endswith('__'):
-            if not hasattr(self, varname):
-                raise AttributeError, "%s has no attribute %s" % (self, varname)
+            raise AttributeError, "%s has no attribute %s" % (self, varname)
             
         if varname != 'hdr':   #  This is unusual, for anything that happens
             val = val[varname] # a lot we should preload at __init__.
@@ -140,7 +141,10 @@ class RPMDBPackageSack(PackageSackBase):
         self._simple_pkgtup_list = []
         self._get_pro_cache = {}
         self._get_req_cache = {}
-        misc.unshare_data()
+        #  We can be called on python shutdown (due to yb.__del__), at which
+        # point other modules might not be available.
+        if misc is not None:
+            misc.unshare_data()
         self._cache = {
             'provides' : { },
             'requires' : { },
@@ -265,12 +269,12 @@ class RPMDBPackageSack(PackageSackBase):
         
         (name, arch) = naTup
 
-        allpkg = [ po.pkgtup for po in self._search(name=name, arch=arch) ]
+        allpkg = self._search(name=name, arch=arch)
 
         if not allpkg:
             raise Errors.PackageSackError, 'No Package Matching %s' % name
 
-        return misc.newestInList(allpkg)
+        return [ po.pkgtup for po in misc.newestInList(allpkg) ]
 
     def returnNewestByName(self, name=None):
         if not name:
@@ -319,6 +323,9 @@ class RPMDBPackageSack(PackageSackBase):
         return False
 
     def returnPackages(self, repoid=None, patterns=None, ignore_case=False):
+        """Returns a list of packages. Note that the packages are
+           always filtered to those matching the patterns/case. repoid is
+           ignored, and is just here for compatibility with non-rpmdb sacks. """
         if not self._completely_loaded:
             rpats = self._compile_patterns(patterns, ignore_case)
             for hdr, idx in self._all_packages():
@@ -332,6 +339,53 @@ class RPMDBPackageSack(PackageSackBase):
             pkgobjlist = pkgobjlist[0] + pkgobjlist[1]
         return pkgobjlist
 
+    def simpleVersion(self, main_only=False, groups={}):
+        """ Return a simple version for all installed packages. """
+        def _up_revs(irepos, repoid, rev, pkg, csum):
+            irevs = irepos.setdefault(repoid, {})
+            rpsv = irevs.setdefault(None, PackageSackVersion())
+            rpsv.update(pkg, csum)
+            if rev is not None:
+                rpsv = irevs.setdefault(rev, PackageSackVersion())
+                rpsv.update(pkg, csum)
+
+        main = PackageSackVersion()
+        irepos = {}
+        main_grps = {}
+        irepos_grps = {}
+        for pkg in sorted(self.returnPackages()):
+            ydbi = pkg.yumdb_info
+            csum = None
+            if 'checksum_type' in ydbi and 'checksum_data' in ydbi:
+                csum = (ydbi.checksum_type, ydbi.checksum_data)
+            main.update(pkg, csum)
+
+            for group in groups:
+                if pkg.name in groups[group]:
+                    if group not in main_grps:
+                        main_grps[group] = PackageSackVersion()
+                        irepos_grps[group] = {}
+                    main_grps[group].update(pkg, csum)
+
+            if main_only:
+                continue
+
+            repoid = 'installed'
+            rev = None
+            if 'from_repo' in pkg.yumdb_info:
+                repoid = '@' + pkg.yumdb_info.from_repo
+                if 'from_repo_revision' in pkg.yumdb_info:
+                    rev = pkg.yumdb_info.from_repo_revision
+
+            _up_revs(irepos, repoid, rev, pkg, csum)
+            for group in groups:
+                if pkg.name in groups[group]:
+                    _up_revs(irepos_grps[group], repoid, rev, pkg, csum)
+
+        if groups:
+            return [main, irepos, main_grps, irepos_grps]
+        return [main, irepos]
+
     @staticmethod
     def _find_search_fields(fields, searchstrings, hdr):
         count = 0
@@ -380,7 +434,11 @@ class RPMDBPackageSack(PackageSackBase):
 
     def excludeArchs(self, archlist):
         pass
-
+    
+    def returnLeafNodes(self, repoid=None):
+        ts = self.readOnlyTS()
+        return [ self._makePackageObject(h, mi) for (h, mi) in ts.returnLeafNodes(headers=True) ]
+        
     # Helper functions
     def _all_packages(self):
         '''Generator that yield (header, index) for all packages
@@ -419,8 +477,7 @@ class RPMDBPackageSack(PackageSackBase):
 
 
     def _search(self, name=None, epoch=None, ver=None, rel=None, arch=None):
-        '''Generator that yields matching packages
-        '''
+        '''List of matching packages, to zero or more of NEVRA.'''
         pkgtup = (name, arch, epoch, ver, rel)
         if self._tup2pkg.has_key(pkgtup):
             return [self._tup2pkg[pkgtup]]
@@ -735,6 +792,9 @@ class RPMDBAdditionalDataPackage(object):
     def _read(self, attr):
         attr = _sanitize(attr)
 
+        if attr.endswith('.tmp'):
+            raise AttributeError, "%s has no attribute %s" % (self, attr)
+
         if attr in self._read_cached_data:
             return self._read_cached_data[attr]
 
@@ -751,7 +811,7 @@ class RPMDBAdditionalDataPackage(object):
     def _delete(self, attr):
         """remove the attribute file"""
 
-        self.get(attr)
+        attr = _sanitize(attr)
         fn = self._mydir + '/' + attr
         if attr in self._read_cached_data:
             del self._read_cached_data[attr]
@@ -776,18 +836,20 @@ class RPMDBAdditionalDataPackage(object):
         else:
             object.__delattr__(self, attr)
 
-    def __iter__(self):
+    def __iter__(self, show_hidden=False):
         for item in self._read_cached_data:
             yield item
         for item in glob.glob(self._mydir + '/*'):
             item = item[(len(self._mydir) + 1):]
             if item in self._read_cached_data:
                 continue
+            if not show_hidden and item.endswith('.tmp'):
+                continue
             yield item
 
     def clean(self):
         # purge out everything
-        for item in self:
+        for item in self.__iter__(show_hidden=True):
             self._delete(item)
         try:
             os.rmdir(self._mydir)
