@@ -25,7 +25,7 @@ import types
 import sys
 from yum.constants import *
 from yum import _
-
+import misc
 
 class NoOutputCallBack:
     def __init__(self):
@@ -138,12 +138,38 @@ class SimpleCliCallBack(RPMBaseCallback):
         if msgs:
             print msgs,
 
+#  This is ugly, but atm. rpm can go insane and run the "cleanup" phase
+# without the "install" phase if it gets an exception in it's callback. The
+# following means that we don't really need to know/care about that in the
+# display callback functions.
+#  Note try/except's in RPMTransaction are for the same reason.
+class _WrapNoExceptions:
+    def __init__(self, parent):
+        self.__parent = parent
+
+    def __getattr__(self, name):
+        """ Wraps all access to the parent functions. This is so it'll eat all
+            exceptions because rpm doesn't like exceptions in the callback. """
+        func = getattr(self.__parent, name)
+
+        def newFunc(*args, **kwargs):
+            try:
+                func(*args, **kwargs)
+            except:
+                pass
+
+        newFunc.__name__ = func.__name__
+        newFunc.__doc__ = func.__doc__
+        newFunc.__dict__.update(func.__dict__)
+        return newFunc
+
 class RPMTransaction:
     def __init__(self, base, test=False, display=NoOutputCallBack):
         if not callable(display):
             self.display = display
         else:
             self.display = display() # display callback
+        self.display = _WrapNoExceptions(self.display)
         self.base = base # base yum object b/c we need so much
         self.test = test # are we a test?
         self.trans_running = False
@@ -160,14 +186,29 @@ class RPMTransaction:
         if not os.path.exists(self.base.conf.persistdir):
             os.makedirs(self.base.conf.persistdir) # make the dir, just in case
 
+    # Error checking? -- these should probably be where else
+    def _fdSetNonblock(self, fd):
+        """ Set the Non-blocking flag for a filedescriptor. """
+        flag = os.O_NONBLOCK
+        current_flags = fcntl.fcntl(fd, fcntl.F_GETFL)
+        if current_flags & flag:
+            return
+        fcntl.fcntl(fd, fcntl.F_SETFL, current_flags | flag)
+
+    def _fdSetCloseOnExec(self, fd):
+        """ Set the close on exec. flag for a filedescriptor. """
+        flag = fcntl.FD_CLOEXEC
+        current_flags = fcntl.fcntl(fd, fcntl.F_GETFD)
+        if current_flags & flag:
+            return
+        fcntl.fcntl(fd, fcntl.F_SETFD, current_flags | flag)
+
     def _setupOutputLogging(self):
         # UGLY... set up the transaction to record output from scriptlets
         (r, w) = os.pipe()
         # need fd objects, and read should be non-blocking
         self._readpipe = os.fdopen(r, 'r')
-        fcntl.fcntl(self._readpipe.fileno(), fcntl.F_SETFL,
-                    fcntl.fcntl(self._readpipe.fileno(),
-                                fcntl.F_GETFL) | os.O_NONBLOCK)
+        self._fdSetNonblock(self._readpipe.fileno())
         self._writepipe = os.fdopen(w, 'w')
         self.base.ts.scriptFd = self._writepipe.fileno()
         rpm.setVerbosity(rpm.RPMLOG_INFO)
@@ -189,6 +230,11 @@ class RPMTransaction:
         except IOError:
             pass
 
+    def _scriptout(self, data):
+        msgs = self._scriptOutput()
+        self.display.scriptout(data, msgs)
+        self.base.history.log_scriptlet_output(data, msgs)
+
     def __del__(self):
         self._shutdownOutputLogging()
         
@@ -211,14 +257,14 @@ class RPMTransaction:
         if self.test: return
     
         if not hasattr(self, '_ts_done'):
-            # need config variable to put this in a better path/name
-            te_fn = '%s/transaction-done.%s' % (self.base.conf.persistdir, self._ts_time)
-            self.ts_done_fn = te_fn
+            self.ts_done_fn = '%s/transaction-done.%s' % (self.base.conf.persistdir, self._ts_time)
+            
             try:
-                self._ts_done = open(te_fn, 'w')
+                self._ts_done = open(self.ts_done_fn, 'w')
             except (IOError, OSError), e:
                 self.display.errorlog('could not open ts_done file: %s' % e)
                 return
+            self._fdSetCloseOnExec(self._ts_done.fileno())
         
         # walk back through self._te_tuples
         # make sure the package and the action make some kind of sense
@@ -255,8 +301,14 @@ class RPMTransaction:
         # hope springs eternal that this isn't wrong
         msg = '%s %s:%s-%s-%s.%s\n' % (t,e,n,v,r,a)
 
-        self._ts_done.write(msg)
-        self._ts_done.flush()
+        try:
+            self._ts_done.write(msg)
+            self._ts_done.flush()
+        except (IOError, OSError), e:
+            #  Having incomplete transactions is probably worse than having
+            # nothing.
+            del self._ts_done
+            misc.unlink_f(self.ts_done_fn)
         self._te_tuples.pop(0)
     
     def ts_all(self):
@@ -288,20 +340,30 @@ class RPMTransaction:
         self._ts_time = time.strftime('%Y-%m-%d.%H:%M.%S')
         tsfn = '%s/transaction-all.%s' % (self.base.conf.persistdir, self._ts_time)
         self.ts_all_fn = tsfn
-        if not os.path.exists(self.base.conf.persistdir):
-            os.makedirs(self.base.conf.persistdir) # make the dir,
+        # to handle us being inside a chroot at this point
+        # we hand back the right path to those 'outside' of the chroot() calls
+        # but we're using the right path inside.
+        if self.base.conf.installroot != '/':
+            tsfn = tsfn.replace(os.path.normpath(self.base.conf.installroot),'')
         try:
+            if not os.path.exists(os.path.dirname(tsfn)):
+                os.makedirs(os.path.dirname(tsfn)) # make the dir,
             fo = open(tsfn, 'w')
         except (IOError, OSError), e:
             self.display.errorlog('could not open ts_all file: %s' % e)
             return
 
-        for (t,e,n,v,r,a) in self._te_tuples:
-            msg = "%s %s:%s-%s-%s.%s\n" % (t,e,n,v,r,a)
-            fo.write(msg)
-        fo.flush()
-        fo.close()
-    
+        try:
+            for (t,e,n,v,r,a) in self._te_tuples:
+                msg = "%s %s:%s-%s-%s.%s\n" % (t,e,n,v,r,a)
+                fo.write(msg)
+            fo.flush()
+            fo.close()
+        except (IOError, OSError), e:
+            #  Having incomplete transactions is probably worse than having
+            # nothing.
+            misc.unlink_f(tsfn)
+
     def callback( self, what, bytes, total, h, user ):
         if what == rpm.RPMCALLBACK_TRANS_START:
             self._transStart( bytes, total, h )
@@ -331,6 +393,9 @@ class RPMTransaction:
             self._cpioError(bytes, total, h)
         elif what == rpm.RPMCALLBACK_UNPACK_ERROR:
             self._unpackError(bytes, total, h)
+        # SCRIPT_ERROR is only in rpm >= 4.6.0
+        elif hasattr(rpm, "RPMCALLBACK_SCRIPT_ERROR") and what == rpm.RPMCALLBACK_SCRIPT_ERROR:
+            self._scriptError(bytes, total, h)
     
     
     def _transStart(self, bytes, total, h):
@@ -350,22 +415,26 @@ class RPMTransaction:
         self.lastmsg = None
         hdr = None
         if h is not None:
-            hdr, rpmloc = h
+            hdr, rpmloc = h[0], h[1]
             handle = self._makeHandle(hdr)
-            fd = os.open(rpmloc, os.O_RDONLY)
-            self.filehandles[handle]=fd
-            if self.trans_running:
-                self.total_installed += 1
-                self.complete_actions += 1
-                self.installed_pkg_names.append(hdr['name'])
-            return fd
+            try:
+                fd = os.open(rpmloc, os.O_RDONLY)
+            except OSError, e:
+                self.displaylog.errorlog("Error: Cannot open file %s: %s" % (rpmloc, e))
+            else:
+                self.filehandles[handle]=fd
+                if self.trans_running:
+                    self.total_installed += 1
+                    self.complete_actions += 1
+                    self.installed_pkg_names.append(hdr['name'])
+                return fd
         else:
             self.display.errorlog("Error: No Header to INST_OPEN_FILE")
             
     def _instCloseFile(self, bytes, total, h):
         hdr = None
         if h is not None:
-            hdr, rpmloc = h
+            hdr, rpmloc = h[0], h[1]
             handle = self._makeHandle(hdr)
             os.close(self.filehandles[handle])
             fd = 0
@@ -375,10 +444,14 @@ class RPMTransaction:
                 txmbrs = self.base.tsInfo.getMembers(pkgtup=pkgtup)
                 for txmbr in txmbrs:
                     self.display.filelog(txmbr.po, txmbr.output_state)
-                    self.display.scriptout(txmbr.po, self._scriptOutput())
+                    self._scriptout(txmbr.po)
+                    # NOTE: We only do this for install, not erase atm.
+                    #       because we don't get pkgtup data for erase (this 
+                    #       includes "Updated" pkgs).
+                    pid   = self.base.history.pkg2pid(txmbr.po)
+                    state = self.base.history.txmbr2state(txmbr)
+                    self.base.history.trans_data_pid_end(pid, state)
                     self.ts_done(txmbr.po, txmbr.output_state)
-                
-                
     
     def _instProgress(self, bytes, total, h):
         if h is not None:
@@ -389,7 +462,7 @@ class RPMTransaction:
                                 self.complete_actions, self.total_actions)
 
             else:
-                hdr, rpmloc = h
+                hdr, rpmloc = h[0], h[1]
                 pkgtup = self._dopkgtup(hdr)
                 txmbrs = self.base.tsInfo.getMembers(pkgtup=pkgtup)
                 for txmbr in txmbrs:
@@ -413,7 +486,7 @@ class RPMTransaction:
         
         self.display.event(h, action, 100, 100, self.complete_actions,
                             self.total_actions)
-        self.display.scriptout(h, self._scriptOutput())
+        self._scriptout(h)
         
         if self.test: return # and we're done
         self.ts_done(h, action)
@@ -429,21 +502,56 @@ class RPMTransaction:
         pass
         
     def _cpioError(self, bytes, total, h):
-        (hdr, rpmloc) = h
+        hdr, rpmloc = h[0], h[1]
         pkgtup = self._dopkgtup(hdr)
         txmbrs = self.base.tsInfo.getMembers(pkgtup=pkgtup)
         for txmbr in txmbrs:
             msg = "Error in cpio payload of rpm package %s" % txmbr.po
+            txmbr.output_state = TS_FAILED
             self.display.errorlog(msg)
             # FIXME - what else should we do here? raise a failure and abort?
     
     def _unpackError(self, bytes, total, h):
-        (hdr, rpmloc) = h
+        hdr, rpmloc = h[0], h[1]
         pkgtup = self._dopkgtup(hdr)
         txmbrs = self.base.tsInfo.getMembers(pkgtup=pkgtup)
         for txmbr in txmbrs:
+            txmbr.output_state = TS_FAILED
             msg = "Error unpacking rpm package %s" % txmbr.po
             self.display.errorlog(msg)
             # FIXME - should we raise? I need a test case pkg to see what the
             # right behavior should be
                 
+    def _scriptError(self, bytes, total, h):
+        if not isinstance(h, types.TupleType):
+            # fun with install/erase transactions, see rhbz#484729
+            h = (h, None)
+        hdr, rpmloc = h[0], h[1]
+        remove_hdr = False # if we're in a clean up/remove then hdr will not be an rpm.hdr
+        if not isinstance(hdr, rpm.hdr):
+            txmbrs = [hdr]
+            remove_hdr = True
+        else:
+            pkgtup = self._dopkgtup(hdr)
+            txmbrs = self.base.tsInfo.getMembers(pkgtup=pkgtup)
+            
+        for pkg in txmbrs:
+            # "bytes" carries the failed scriptlet tag,
+            # "total" carries fatal/non-fatal status
+            scriptlet_name = rpm.tagnames.get(bytes, "<unknown>")
+            if remove_hdr:
+                package_name = pkg
+            else:
+                package_name = pkg.po
+                
+            if total:
+                msg = ("Error in %s scriptlet in rpm package %s" % 
+                        (scriptlet_name, package_name))
+                if not remove_hdr:        
+                    pkg.output_state = TS_FAILED
+            else:
+                msg = ("Non-fatal %s scriptlet failure in rpm package %s" % 
+                       (scriptlet_name, package_name))
+            self.display.errorlog(msg)
+            # FIXME - what else should we do here? raise a failure and abort?
+    
