@@ -36,6 +36,7 @@ import operator
 from yum.misc import seq_max_split
 from yum.i18n import to_utf8, to_unicode
 import sys
+import re
 
 def catchSqliteException(func):
     """This decorator converts sqlite exceptions into RepoError"""
@@ -60,6 +61,118 @@ def catchSqliteException(func):
 def _share_data(value):
     return misc.share_data(value)
 
+# FIXME: parsePackages()
+def _parse_pkg_n(match, regexp_match, n):
+    if match == n:
+        return True
+    if not regexp_match:
+        return False
+
+    if (match and n and match[0] not in ('?', '*') and match[0] != n[0]):
+        return False
+    if regexp_match(n):
+        return True
+    return False
+
+def _parse_pkg(match, regexp_match, data, e,v,r,a):
+
+    n = data['n']
+    assert e, 'Nothing in epoch'
+    # Worthless speed hacks?
+    if match == n:
+        return True
+    if (match and n and match[0] not in ('?', '*') and
+        match[0] != n[0] and match[0] != e[0]):
+        return False
+
+    if 'nameArch' not in data:
+        data['nameArch'] = '%s.%s' % (n, a)
+        data['nameVerRelArch'] = '%s-%s-%s.%s' % (n, v, r, a)
+        data['nameVer'] = '%s-%s' % (n, v)
+        data['nameVerRel'] = '%s-%s-%s' % (n, v, r)
+        data['envra'] = '%s:%s-%s-%s.%s' % (e, n, v, r, a)
+        data['nevra'] = '%s-%s:%s-%s.%s' % (n, e, v, r, a)
+    data = set([n, data['nameArch'], data['nameVerRelArch'], data['nameVer'],
+                data['nameVerRel'], data['envra'], data['nevra']])
+
+    if match in data:
+        return True
+    if not regexp_match:
+        return False
+
+    for item in data:
+        if regexp_match(item):
+            return True
+    return False
+
+def _excluder_match(excluder, match, regexp_match, data, e,v,r,a):
+    if False: pass
+    elif excluder in ('eq', 'match'):
+        if _parse_pkg(match, regexp_match, data, e,v,r,a):
+            return True
+
+    elif excluder in ('name.eq', 'name.match'):
+        if _parse_pkg_n(match, regexp_match, data['n']):
+            return True
+
+    elif excluder in ('arch.eq', 'arch.match'):
+        if _parse_pkg_n(match, regexp_match, a):
+            return True
+
+    elif excluder == 'nevr.eq':
+        if 'nevr' not in data:
+            data['nevr'] = '%s-%s:%s-%s' % (data['n'], e, v, r)
+        if match == data['nevr']:
+            return True
+
+    elif excluder in ('nevra.eq', 'nevra.match'):
+        if 'nevra' not in data:
+            data['nevra'] = '%s-%s:%s-%s.%s' % (data['n'], e, v, r, a)
+        if _parse_pkg_n(match, regexp_match, data['nevra']):
+            return True
+
+    elif excluder == 'name.in':
+        if data['n'] in match:
+            return True
+
+    elif excluder == 'nevr.in':
+        if 'nevr' not in data:
+            data['nevr'] = '%s-%s:%s-%s' % (data['n'], e, v, r)
+        if data['nevr'] in match:
+            return True
+
+    elif excluder == 'nevra.in':
+        if 'nevra' not in data:
+            data['nevra'] = '%s-%s:%s-%s.%s' % (data['n'], e, v, r, a)
+        if data['nevra'] in match:
+            return True
+
+    elif excluder == 'pkgtup.eq':
+        if match == data['pkgtup']:
+            return True
+
+    elif excluder == 'pkgtup.in':
+        if data['pkgtup'] in match:
+            return True
+
+    elif excluder == 'marked':
+        if data['marked']:
+            return True
+
+    elif excluder == 'washed':
+        if not data['marked']:
+            return True
+
+    elif excluder == '*':
+        return True
+
+    else:
+        assert False, 'Bad excluder: ' + excluder
+        return None
+
+    return False
+
+
 class YumAvailablePackageSqlite(YumAvailablePackage, PackageObject, RpmBase):
     def __init__(self, repo, db_obj):
         self.prco = { 'obsoletes': (),
@@ -103,13 +216,9 @@ class YumAvailablePackageSqlite(YumAvailablePackage, PackageObject, RpmBase):
             except (IndexError, KeyError):
                 pass
 
-        for item in ['pkgId']:
-            try:
-                setattr(self, item, db_obj[item])
-            except (IndexError, KeyError):
-                pass
-
         try:
+            self.pkgId = db_obj['pkgId']
+
             checksum_type = _share_data(db_obj['checksum_type'])
             check_sum = (checksum_type, db_obj['pkgId'], True)
             self._checksums = [ check_sum ]
@@ -144,9 +253,10 @@ class YumAvailablePackageSqlite(YumAvailablePackage, PackageObject, RpmBase):
                          'checksum_value' : 'pkgId',
                         }
 
+        # If these existed, then we wouldn't get here ... and nothing in the DB
+        # starts and ends with __'s. So these are missing.
         if varname.startswith('__') and varname.endswith('__'):
-            if not hasattr(self, varname):
-                raise AttributeError, varname
+            raise AttributeError, varname
         
         dbname = varname
         if db2simplemap.has_key(varname):
@@ -291,15 +401,22 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         self.primarydb = {}
         self.filelistsdb = {}
         self.otherdb = {}
-        self.excludes = {}
+        self.excludes = {}     # of [repo] => {} of pkgId's => 1
         self._excludes = set() # of (repo, pkgKey)
+        self._exclude_whitelist = set() # of (repo, pkgKey)
         self._all_excludes = {}
         self._search_cache = {
             'provides' : { },
             'requires' : { },
             }
         self._key2pkg = {}
+        self._pkgname2pkgkeys = {}
+        self._pkgtup2pkgs = {}
+        self._pkgnames_loaded = set()
         self._arch_allowed = None
+        self._pkgExcluder = []
+        self._pkgExcludeIds = {}
+        self._pkgobjlist_dirty = False
 
     @catchSqliteException
     def _sql_MD(self, MD, repo, sql, *args):
@@ -315,6 +432,18 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         sql = "SELECT count(pkgId) FROM packages"
         return self._sql_MD('primary', repo, sql).fetchone()[0]
         
+    def _clean_pkgobjlist(self):
+        """ If the pkgobjlist is dirty (possible pkgs on it which are excluded)
+            then clean it, and return the clean list. """
+        assert hasattr(self, 'pkgobjlist')
+
+        if self._pkgobjlist_dirty:
+            pol = filter(lambda x: not self._pkgExcluded(x), self.pkgobjlist)
+            self.pkgobjlist = pol
+            self._pkgobjlist_dirty = False
+
+        return self.pkgobjlist
+
     def __len__(self):
         # First check if everything is excluded
         all_excluded = True
@@ -325,14 +454,13 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         if all_excluded:
             return 0
             
+        if hasattr(self, 'pkgobjlist'):
+            return len(self._clean_pkgobjlist())
+
         exclude_num = 0
         for repo in self.excludes:
             exclude_num += len(self.excludes[repo])
-        if hasattr(self, 'pkgobjlist'):
-            return len(self.pkgobjlist) - exclude_num
-        
         pkg_num = 0
-        sql = "SELECT count(pkgId) FROM packages"
         for repo in self.primarydb:
             pkg_num += self._sql_MD_pkg_num('primary', repo)
         return pkg_num - exclude_num
@@ -344,7 +472,11 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
             del self._memoize_provides
         if hasattr(self, 'pkgobjlist'):
             del self.pkgobjlist
+        self._pkgobjlist_dirty = False
         self._key2pkg = {}
+        self._pkgname2pkgkeys = {}
+        self._pkgnames_loaded = set()
+        self._pkgtup2pkgs = {}
         self._search_cache = {
             'provides' : { },
             'requires' : { },
@@ -364,7 +496,11 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         self.otherdb = {}
         self.excludes = {}
         self._excludes = set()
+        self._exclude_whitelist = set()
         self._all_excludes = {}
+        self._pkgExcluder = []
+        self._pkgExcludeIds = {}
+        self._pkgobjlist_dirty = False
 
         yumRepo.YumPackageSack.close(self)
 
@@ -376,6 +512,18 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
     def _checkIndexes(self, failure='error'):
         return
 
+    def _delPackageRK(self, repo, pkgKey):
+        ''' Exclude a package so that _pkgExcluded*() knows it's gone.
+            Note that this doesn't update self.exclude. '''
+        self._excludes.add((repo, pkgKey))
+        # Don't keep references around, just wastes memory.
+        if repo in self._key2pkg:
+            po = self._key2pkg[repo].pop(pkgKey, None)
+            if po is not None: # Will also be in the pkgtup2pkgs cache...
+                pos = self._pkgtup2pkgs[po.pkgtup]
+                pos = filter(lambda x: id(x) == id(po), pos)
+                self._pkgtup2pkgs[po.pkgtup] = pos
+
     # Remove a package
     # Because we don't want to remove a package from the database we just
     # add it to the exclude list
@@ -383,7 +531,10 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         if not self.excludes.has_key(obj.repo):
             self.excludes[obj.repo] = {}
         self.excludes[obj.repo][obj.pkgId] = 1
-        self._excludes.add( (obj.repo, obj.pkgKey) )
+        if (obj.repo, obj.pkgKey) in self._exclude_whitelist:
+            self._exclude_whitelist.discard((obj.repo, obj.pkgKey))
+        self._delPackageRK(obj.repo, obj.pkgKey)
+        self._pkgobjlist_dirty = True
 
     def _delAllPackages(self, repo):
         """ Exclude all packages from the repo. """
@@ -392,6 +543,8 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
             del self.excludes[repo]
         if repo in self._key2pkg:
             del self._key2pkg[repo]
+        if repo in self._pkgname2pkgkeys:
+            del self._pkgname2pkgkeys[repo]
 
     def _excluded(self, repo, pkgId):
         if repo in self._all_excludes:
@@ -408,42 +561,195 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
 
         return (repo, pkgKey) in self._excludes
 
-    def _pkgArchExcluded(self, pkgarch):
-        """ Test the arch for a package against the archlist we were passed. """
-        if self._arch_allowed is not None and pkgarch not in self._arch_allowed:
+    def _pkgExcludedRKNEVRA(self, repo,pkgKey, n,e,v,r,a):
+        ''' Main function to use for "can we use this package" question.
+                . Tests repo against allowed repos.
+                . Tests pkgKey against allowed packages.
+                . Tests arch against allowed arches.
+                . Tests addPackageExcluder() calls.
+        '''
+
+        if (repo, pkgKey) in self._exclude_whitelist:
+            return False
+
+        if self._pkgKeyExcluded(repo, pkgKey):
             return True
+
+        if self._arch_allowed is not None and a not in self._arch_allowed:
+            self._delPackageRK(repo, pkgKey)
+            return True
+
+        if not self._pkgExcluder:
+            return False
+
+        data = {'n' : n.lower(), 'pkgtup' : (n, a, e, v, r), 'marked' : False}
+        e = e.lower()
+        v = v.lower()
+        r = r.lower()
+        a = a.lower()
+
+        for repoid, excluder, match, regexp_match in self._pkgExcluder:
+            if repoid is not None and repoid != repo.id:
+                continue
+
+            exSPLIT = excluder.split('.', 1)
+            if len(exSPLIT) != 2:
+                assert False, 'Bad excluder: ' + excluder
+                continue
+
+            exT, exM = exSPLIT
+            if False: pass
+            elif exT == 'exclude':
+                if _excluder_match(exM, match, regexp_match, data, e,v,r,a):
+                    self._delPackageRK(repo, pkgKey)
+                    return True
+
+            elif exT == 'include':
+                if _excluder_match(exM, match, regexp_match, data, e,v,r,a):
+                    break
+
+            elif exT == 'mark':
+                if data['marked']:
+                    pass # Speed opt. don't do matches we don't need to do.
+                elif _excluder_match(exM, match, regexp_match, data, e,v,r,a):
+                    data['marked'] = True
+
+            elif exT == 'wash':
+                if not data['marked']:
+                    pass # Speed opt. don't do matches we don't need to do.
+                elif _excluder_match(exM, match, regexp_match, data, e,v,r,a):
+                    data['marked'] = False
+
+            else:
+                assert False, 'Bad excluder: ' + excluder
+
+        self._exclude_whitelist.add((repo, pkgKey))
         return False
 
-    def _pkgExcluded(self, po):
-        return (self._pkgKeyExcluded(po.repo, po.pkgKey) or
-                self._pkgArchExcluded(po.arch))
+    def _pkgExcludedRKT(self, repo,pkgKey, pkgtup):
+        ''' Helper function to call _pkgRKNEVRAExcluded.
+            Takes a repo, pkgKey and a package tuple'''
+        (n,a,e,v,r) = pkgtup
+        return self._pkgExcludedRKNEVRA(repo, pkgKey, n,e,v,r,a)
 
-    def _packageByKey(self, repo, pkgKey):
+    def _pkgExcludedRKD(self, repo,pkgKey, data):
+        ''' Helper function to call _pkgRKNEVRAExcluded.
+            Takes a repo, pkgKey and a dict of package data'''
+        (n,a,e,v,r) = (data['name'], data['arch'],
+                       data['epoch'], data['version'], data['release'])
+        return self._pkgExcludedRKNEVRA(repo, pkgKey, n,e,v,r,a)
+
+    def _pkgExcluded(self, po):
+        ''' Helper function to call _pkgRKNEVRAExcluded.
+            Takes a package object. '''
+        return self._pkgExcludedRKT(po.repo, po.pkgKey, po.pkgtup)
+
+    def addPackageExcluder(self, repoid, excluderid, excluder, *args):
+        """ Add an "excluder" for all packages in the repo/sack. Can basically
+            do anything based on nevra, changes lots of exclude decisions from
+            "preload package; test; delPackage" into "load excluder".
+            Excluderid is used so the caller doesn't have to track
+            "have I loaded the excluder for this repo.", it's probably only
+            useful when repoid is None ... if it turns out utterly worthless
+            then it's still not a huge wart. """
+        if excluderid is not None and excluderid in self._pkgExcludeIds:
+            return
+
+        match        = None
+        regexp_match = None
+        if False: pass
+        elif excluder.endswith('.eq'):
+            assert len(args) == 1
+            match = args[0].lower()
+        elif excluder.endswith('.in'):
+            assert len(args) == 1
+            match = args[0]
+        elif excluder.endswith('.match'):
+            assert len(args) == 1
+            match = args[0].lower()
+            if misc.re_glob(match):
+                regexp_match = re.compile(fnmatch.translate(match)).match
+        elif excluder.endswith('.*'):
+            assert len(args) == 0
+        elif excluder.endswith('.marked'):
+            assert len(args) == 0
+        elif excluder.endswith('.washed'):
+            assert len(args) == 0
+        #  Really need to do this, need to cleanup pkgExcluder first though
+        # or it does nothing.
+        # self._pkgobjlist_dirty = True
+        self._pkgExcluder.append((repoid, excluder, match, regexp_match))
+        if excluderid is not None:
+            self._pkgExcludeIds[excluderid] = len(self._pkgExcluder)
+
+    def _packageByKey(self, repo, pkgKey, exclude=True):
         """ Lookup a pkg by it's pkgKey, if we don't have it load it """
+        # Speed hack, so we don't load the pkg. if the pkgKey is dead.
+        if exclude and self._pkgKeyExcluded(repo, pkgKey):
+            return None
+
         if not self._key2pkg.has_key(repo):
             self._key2pkg[repo] = {}
+            self._pkgname2pkgkeys[repo] = {}
         if not self._key2pkg[repo].has_key(pkgKey):
-            sql = "SELECT pkgKey, pkgId, name, epoch, version, release " \
+            sql = "SELECT pkgKey, pkgId, name, epoch, version, release, arch " \
                   "FROM packages WHERE pkgKey = ?"
             data = self._sql_MD('primary', repo, sql, (pkgKey,)).fetchone()
             if data is None:
                 msg = "pkgKey %s doesn't exist in repo %s" % (pkgKey, repo)
                 raise Errors.RepoError, msg
-
-            self._key2pkg[repo][pkgKey] = self.pc(repo, data)
-        if self._pkgArchExcluded(self._key2pkg[repo][pkgKey].arch):
-            return None
+            if exclude and self._pkgExcludedRKD(repo, pkgKey, data):
+                return None
+            po = self.pc(repo, data)
+            self._key2pkg[repo][pkgKey] = po
+            self._pkgtup2pkgs.setdefault(po.pkgtup, []).append(po)
+            pkgkeys = self._pkgname2pkgkeys[repo].setdefault(data['name'], [])
+            pkgkeys.append(pkgKey)
         return self._key2pkg[repo][pkgKey]
         
-    def _packageByKeyData(self, repo, pkgKey, data):
+    def _packageByKeyData(self, repo, pkgKey, data, exclude=True):
         """ Like _packageByKey() but we already have the data for .pc() """
-        if self._pkgArchExcluded(data['arch']):
+        if exclude and self._pkgExcludedRKD(repo, pkgKey, data):
             return None
+        if repo not in self._key2pkg:
+            self._key2pkg[repo] = {}
+            self._pkgname2pkgkeys[repo] = {}
         if data['pkgKey'] not in self._key2pkg.get(repo, {}):
             po = self.pc(repo, data)
-            self._key2pkg.setdefault(repo, {})[pkgKey] = po
+            self._key2pkg[repo][pkgKey] = po
+            self._pkgtup2pkgs.setdefault(po.pkgtup, []).append(po)
+            pkgkeys = self._pkgname2pkgkeys[repo].setdefault(data['name'], [])
+            pkgkeys.append(pkgKey)
         return self._key2pkg[repo][data['pkgKey']]
 
+    def _pkgtupByKeyData(self, repo, pkgKey, data):
+        """ Like _packageByKeyData() but we don't create the package, we just
+            return the pkgtup. """
+        if self._pkgExcludedRKD(repo, pkgKey, data):
+            return None
+        if repo not in self._key2pkg:
+            self._key2pkg[repo] = {}
+            self._pkgname2pkgkeys[repo] = {}
+        if data['pkgKey'] in self._key2pkg.get(repo, {}):
+            return self._key2pkg[repo][data['pkgKey']].pkgtup
+        return (data['name'], data['arch'],
+                data['epoch'], data['version'], data['release'])
+
+    def _packagesByName(self, pkgname):
+        """ Load all pkgnames from cache, with a given name. """
+        ret = []
+        for repo in self.primarydb:
+            pkgkeys = self._pkgname2pkgkeys.get(repo, {}).get(pkgname, [])
+            if not pkgkeys:
+                continue
+
+            for pkgkey in pkgkeys:
+                pkg = self._packageByKey(repo, pkgkey)
+                if pkg is None:
+                    continue
+                ret.append(pkg)
+        return ret
+
     def addDict(self, repo, datatype, dataobj, callback=None):
         if self.added.has_key(repo):
             if datatype in self.added[repo]:
@@ -481,8 +787,6 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         """ Takes a cursor and maps the pkgKey rows into a list of packages. """
         if pkgs is None: pkgs = []
         for ob in cur:
-            if self._pkgKeyExcluded(repo, ob['pkgKey']):
-                continue
             if have_data:
                 pkg = self._packageByKeyData(repo, ob['pkgKey'], ob)
             else:
@@ -527,6 +831,22 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
                 break
         return skip_all
 
+    @catchSqliteException
+    def _search_primary_files(self, name):
+        querytype = 'glob'
+        if not misc.re_glob(name):
+            querytype = '='        
+        results = []
+        
+        for (rep,cache) in self.primarydb.items():
+            if rep in self._all_excludes:
+                continue
+            cur = cache.cursor()
+            executeSQL(cur, "select DISTINCT pkgKey from files where name %s ?" % querytype, (name,))
+            self._sql_pkgKey2po(rep, cur, results)
+
+        return misc.unique(results)
+        
     @catchSqliteException
     def searchFiles(self, name, strict=False):
         """search primary if file will be in there, if not, search filelists, use globs, if possible"""
@@ -539,16 +859,28 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         # if so, just use those for the lookup
         
         glob = True
+        file_glob = True
         querytype = 'glob'
+        dirname  = os.path.dirname(name)
+        filename = os.path.basename(name)
         if strict or not misc.re_glob(name):
             glob = False
+            file_glob = False
             querytype = '='
+        elif not misc.re_glob(filename):
+            file_glob = False
 
         # Take off the trailing slash to act like rpm
         if name[-1] == '/':
             name = name[:-1]
        
         pkgs = []
+
+        # ultra simple optimization 
+        if misc.re_primary_filename(name):
+            if not misc.re_glob(dirname): # is the dirname a glob?
+                return self._search_primary_files(name)
+        
         if len(self.filelistsdb) == 0:
             # grab repo object from primarydb and force filelists population in this sack using repo
             # sack.populate(repo, mdtype, callback, cacheonly)
@@ -566,41 +898,71 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
             if pri_pkgs != fil_pkgs:
                 raise Errors.RepoError
 
+        sql_params = []
+        dirname_check = ""
+        if not glob:
+            (pattern, esc) = self._sql_esc(filename)
+            dirname_check = "dirname = ? and filenames LIKE ? %s and " % esc
+            sql_params.append(dirname)
+            sql_params.append('%' + pattern + '%')
+        elif not file_glob:
+            (pattern, esc) = self._sql_esc(filename)
+            dirname_check = "dirname GLOB ? and filenames LIKE ? %s and " % esc
+            sql_params.append(dirname)
+            sql_params.append('%' + pattern + '%')
+        elif filename == '*':
+            # We only care about matching on dirname...
+            for (rep,cache) in self.filelistsdb.items():
+                if rep in self._all_excludes:
+                    continue
+
+                cur = cache.cursor()
+                sql_params.append(dirname)
+                executeSQL(cur, """SELECT pkgKey FROM filelist
+                                   WHERE dirname %s ?""" % (querytype,),
+                           sql_params)
+                self._sql_pkgKey2po(rep, cur, pkgs)
+
+            return misc.unique(pkgs)
+
         for (rep,cache) in self.filelistsdb.items():
             if rep in self._all_excludes:
                 continue
 
             cur = cache.cursor()
 
-            if glob:
-                dirname_check = ""
-            else:
-                dirname = os.path.dirname(name)
-                dirname_check = "dirname = \"%s\" and " % dirname
-
             # grab the entries that are a single file in the 
             # filenames section, use sqlites globbing if it is a glob
             executeSQL(cur, "select pkgKey from filelist where \
                     %s length(filetypes) = 1 and \
                     dirname || ? || filenames \
-                    %s ?" % (dirname_check, querytype), ('/', name))
+                    %s ?" % (dirname_check, querytype), sql_params + ['/',name])
             self._sql_pkgKey2po(rep, cur, pkgs)
 
-            def filelist_globber(dirname, filenames):
-                files = filenames.split('/')
-                fns = map(lambda f: '%s/%s' % (dirname, f), files)
-                if glob:
-                    matches = fnmatch.filter(fns, name)
-                else:
-                    matches = filter(lambda x: name==x, fns)
-                return len(matches)
+            if file_glob:
+                name_re = re.compile(fnmatch.translate(name))
+            def filelist_globber(sql_dirname, sql_filenames):
+                # Note: Can't return bool, because sqlite doesn't like it in
+                #       weird ways. Test:
+                #                         install '*bin/autoheader'
+                #                         provides /lib/security/pam_loginuid.so
+                files = sql_filenames.split('/')
+                if not file_glob:
+                    return int(filename in files)
+
+                fns = map(lambda f: '%s/%s' % (sql_dirname, f), files)
+                for match in fns:
+                    if name_re.match(match):
+                        return 1
+                return 0
 
             cache.create_function("filelist_globber", 2, filelist_globber)
             # for all the ones where filenames is multiple files, 
             # make the files up whole and use python's globbing method
             executeSQL(cur, "select pkgKey from filelist where \
                              %s length(filetypes) > 1 \
-                             and filelist_globber(dirname,filenames)" % dirname_check)
+                             and filelist_globber(dirname,filenames)" % dirname_check,
+                       sql_params)
 
             self._sql_pkgKey2po(rep, cur, pkgs)
 
@@ -676,8 +1038,6 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
             cur = cache.cursor()
             executeSQL(cur, totalstring)
             for ob in cur:
-                if self._pkgKeyExcluded(rep, ob['pkgKey']):
-                    continue
                 pkg = self._packageByKey(rep, ob['pkgKey'])
                 if pkg is None:
                     continue
@@ -704,16 +1064,12 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
                 obsoletes.flags as oflags\
                 from obsoletes,packages where obsoletes.pkgKey = packages.pkgKey")
             for ob in cur:
-                # If the package that is causing the obsoletes is excluded
-                # continue without processing the obsoletes
-                if self._pkgKeyExcluded(rep, ob['pkgKey']):
-                    continue
-                if self._pkgArchExcluded(ob['arch']):
-                    continue
-                    
                 key = ( _share_data(ob['name']), _share_data(ob['arch']),
                         _share_data(ob['epoch']), _share_data(ob['version']),
                         _share_data(ob['release']))
+                if self._pkgExcludedRKT(rep, ob['pkgKey'], key):
+                    continue
+
                 (n,f,e,v,r) = ( _share_data(ob['oname']),
                                 _share_data(ob['oflags']),
                                 _share_data(ob['oepoch']),
@@ -817,8 +1173,6 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
                     if rpmUtils.miscutils.rangeCompare(req, val):
                         tmp.setdefault(pkgkey, []).append(val)
                 for pkgKey, hits in tmp.iteritems():
-                    if self._pkgKeyExcluded(rep, pkgKey):
-                        continue
                     pkg = self._packageByKey(rep, pkgKey)
                     if pkg is None:
                         continue
@@ -840,8 +1194,6 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
                 if rpmUtils.miscutils.rangeCompare(req, val):
                     tmp.setdefault(x['pkgKey'], []).append(val)
             for pkgKey, hits in tmp.iteritems():
-                if self._pkgKeyExcluded(rep, pkgKey):
-                    continue
                 pkg = self._packageByKey(rep, pkgKey)
                 if pkg is None:
                     continue
@@ -862,20 +1214,10 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
             return result
 
         # If it is a filename, search the primary.xml file info
-        for (rep,cache) in self.primarydb.items():
-            if rep in self._all_excludes:
-                continue
-
-            cur = cache.cursor()
-            executeSQL(cur, "select DISTINCT pkgKey from files where name = ?", (name,))
-            for ob in cur:
-                if self._pkgKeyExcluded(rep, ob['pkgKey']):
-                    continue
-                pkg = self._packageByKey(rep, ob['pkgKey'])
-                if pkg is None:
-                    continue
-                result[pkg] = [(name, None, None)]
-        self._search_cache[prcotype][req] = result
+        
+        for pkg in self._search_primary_files(name):
+            result[pkg] = [(name, None, None)]
+            self._search_cache[prcotype][req] = result
         return result
 
     def getProvides(self, name, flags=None, version=(None, None, None)):
@@ -892,23 +1234,25 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         if self._skip_all():
             return []
         
+        loaded_all_names = hasattr(self, 'pkgobjlist')
         returnList = []
-        if hasattr(self, 'pkgobjlist'):
-            names = set(names)
-            for po in self.pkgobjlist:
-                if po.name not in names:
-                    continue
-                if self._pkgExcluded(po):
-                    continue
-                returnList.append(po)
+        user_names = set(names)
+        names = []
+        for pkgname in user_names:
+            if loaded_all_names or pkgname in self._pkgnames_loaded:
+                returnList.extend(self._packagesByName(pkgname))
+            else:
+                names.append(pkgname)
+
+        if not names:
             return returnList
 
         max_entries = constants.PATTERNS_INDEXED_MAX
         if len(names) > max_entries:
-            returnList = set() # Unique
+            # Unique is done at user_names time, above.
             for names in seq_max_split(names, max_entries):
-                returnList.update(self.searchNames(names))
-            return list(returnList)
+                returnList.extend(self.searchNames(names))
+            return returnList
 
         pat_sqls = []
         qsql = """select pkgId,pkgKey,name,epoch,version,release,arch
@@ -919,10 +1263,13 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
 
         for (repo, cache) in self.primarydb.items():
             cur = cache.cursor()
-            executeSQL(cur, qsql, list(names))
+            executeSQL(cur, qsql, names)
 
             self._sql_pkgKey2po(repo, cur, returnList, have_data=True)
 
+        # Mark all the processed pkgnames as fully loaded
+        self._pkgnames_loaded.update([name for name in names])
+
         return returnList
  
     @catchSqliteException
@@ -947,14 +1294,11 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         # If it's not a provides or a filename, we are done
         if prcotype != "provides":
             return results
-        if name[0] != '/' and (not glob or not misc.re_glob(name[0])):
+        if not misc.re_filename(name):
             return results
 
         # If it is a filename, search the primary.xml file info
-        for (rep,cache) in self.primarydb.items():
-            cur = cache.cursor()
-            executeSQL(cur, "select DISTINCT pkgKey from files where name %s ?" % querytype, (name,))
-            self._sql_pkgKey2po(rep, cur, results)
+        results.extend(self._search_primary_files(name))
 
         # if its in the primary.xml files then skip the other check
         if misc.re_primary_filename(name) and not glob:
@@ -1129,16 +1473,13 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         unmatched = misc.unique(unmatched)
         return exactmatch, matched, unmatched
 
-    @catchSqliteException
-    def _buildPkgObjList(self, repoid=None, patterns=None, ignore_case=False):
-        """Builds a list of packages, only containing nevra information. No
-           excludes are done at this stage. """
+    def _setupPkgObjList(self, repoid=None, patterns=None, ignore_case=False):
+        """Setup need_full and patterns for _yieldSQLDataList, also see if
+           we can get away with just using searchNames(). """
 
         if patterns is None:
             patterns = []
 
-        returnList = []
-        
         fields = ['name', 'sql_nameArch', 'sql_nameVerRelArch',
                   'sql_nameVer', 'sql_nameVerRel',
                   'sql_envra', 'sql_nevra']
@@ -1158,12 +1499,22 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
             patterns = self._sql_esc_glob(patterns)
         else:
             tmp = []
+            need_glob = False
             for pat in patterns:
                 if misc.re_glob(pat):
                     tmp.append((pat, 'glob'))
+                    need_glob = True
                 else:
                     tmp.append((pat, '='))
+            if not need_full and not need_glob and patterns:
+                return (need_full, patterns, fields, True)
             patterns = tmp
+        return (need_full, patterns, fields, False)
+
+    @catchSqliteException
+    def _yieldSQLDataList(self, repoid, patterns, fields, ignore_case):
+        """Yields all the package data for the given params. Excludes are done
+           at this stage. """
 
         for (repo,cache) in self.primarydb.items():
             if (repoid == None or repoid == repo.id):
@@ -1184,45 +1535,97 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
                 if pat_sqls:
                     qsql = _FULL_PARSE_QUERY_BEG + " OR ".join(pat_sqls)
                 executeSQL(cur, qsql, pat_data)
-                #  Note: Not using _sql_pkgKey2po() so that we can "un-exclude"
-                # things later on ... if that matters.
                 for x in cur:
-                    po = self._packageByKeyData(repo, x['pkgKey'], x)
-                    if po is None: # Arch exclude is done here.
-                        continue
-                    returnList.append(po)
-        if not patterns:
+                    yield (repo, x)
+
+    def _buildPkgObjList(self, repoid=None, patterns=None, ignore_case=False):
+        """Builds a list of packages, only containing nevra information.
+           Excludes are done at this stage. """
+
+        returnList = []
+
+        data = self._setupPkgObjList(repoid, patterns, ignore_case)
+        (need_full, patterns, fields, names) = data
+        if names:
+            return self.searchNames(patterns)
+
+        for (repo, x) in self._yieldSQLDataList(repoid, patterns, fields,
+                                                ignore_case):
+            po = self._packageByKeyData(repo, x['pkgKey'], x)
+            if po is None:
+                continue
+            returnList.append(po)
+        if not patterns and repoid is None:
             self.pkgobjlist = returnList
+            self._pkgnames_loaded = set() # Save memory
+        if not need_full and repoid is None:
+            # Mark all the processed pkgnames as fully loaded
+            self._pkgnames_loaded.update([po.name for po in returnList])
+
         return returnList
                 
     def returnPackages(self, repoid=None, patterns=None, ignore_case=False):
         """Returns a list of packages, only containing nevra information. The
-           packages are processed for excludes. Note that patterns is just
-           a hint, we are free it ignore it. """
+           packages are processed for excludes. Note that the packages are
+           always filtered to those matching the patterns/case. """
 
         if self._skip_all():
             return []
 
-        if hasattr(self, 'pkgobjlist'):
-            pkgobjlist = self.pkgobjlist
+        internal_pkgoblist = hasattr(self, 'pkgobjlist')
+        if internal_pkgoblist:
+            pkgobjlist = self._clean_pkgobjlist()
         else:
             pkgobjlist = self._buildPkgObjList(repoid, patterns, ignore_case)
+            internal_pkgoblist = hasattr(self, 'pkgobjlist')
 
-        if hasattr(self, 'pkgobjlist') and patterns:
+        if internal_pkgoblist and patterns:
+            internal_pkgoblist = False
             pkgobjlist = parsePackages(pkgobjlist, patterns, not ignore_case,
                                        unique='repo-pkgkey')
             pkgobjlist = pkgobjlist[0] + pkgobjlist[1]
 
+        # Can't unexclude things, and new excludes are done above...
+        if repoid is None:
+            if internal_pkgoblist:
+                pkgobjlist = pkgobjlist[:]
+            return pkgobjlist
+
         returnList = []
         for po in pkgobjlist:
-            if repoid is not None and repoid != po.repoid:
-                continue
-            if self._pkgExcluded(po):
+            if repoid != po.repoid:
                 continue
             returnList.append(po)
 
         return returnList
 
+    def simplePkgList(self, patterns=None, ignore_case=False):
+        """Returns a list of pkg tuples (n, a, e, v, r), optionally from a
+           single repoid. Note that the packages are always filtered to those
+           matching the patterns/case. """
+
+        if self._skip_all():
+            return []
+
+        internal_pkgoblist = hasattr(self, 'pkgobjlist')
+        if internal_pkgoblist:
+            return yumRepo.YumPackageSack.simplePkgList(self, patterns,
+                                                        ignore_case)
+
+        repoid = None
+        returnList = []
+        # Haven't loaded everything, so _just_ get the pkgtups...
+        data = self._setupPkgObjList(repoid, patterns, ignore_case)
+        (need_full, patterns, fields, names) = data
+        for (repo, x) in self._yieldSQLDataList(repoid, patterns, fields,
+                                                ignore_case):
+            # NOTE: Can't unexclude things...
+            pkgtup = self._pkgtupByKeyData(repo, x['pkgKey'], x)
+            if pkgtup is None:
+                continue
+            returnList.append(pkgtup)
+        return returnList
+
     @catchSqliteException
     def searchNevra(self, name=None, epoch=None, ver=None, rel=None, arch=None):        
         """return list of pkgobjects matching the nevra requested"""
@@ -1231,6 +1634,18 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
 
         returnList = []
         
+        if name: # Almost always true...
+            for pkg in self.searchNames(names=[name]):
+                match = True
+                for (col, var) in [('epoch', epoch), ('version', ver),
+                                   ('arch', arch), ('release', rel)]:
+                    if var and getattr(pkg, col) != var:
+                        match = False
+                        break
+                if match:
+                    returnList.append(pkg)
+            return returnList
+
         # make sure some dumbass didn't pass us NOTHING to search on
         empty = True
         for arg in (name, epoch, ver, rel, arch):
