@@ -19,6 +19,7 @@ import time
 import types
 import urlparse
 urlparse.uses_fragment.append("media")
+import gzip
 
 import Errors
 from urlgrabber.grabber import URLGrabber
@@ -33,6 +34,7 @@ import sqlitecachec
 import sqlitesack
 from yum import config
 from yum import misc
+from yum import comps
 from constants import *
 import metalink
 
@@ -178,6 +180,7 @@ class YumPackageSack(packageSack.PackageSack):
                 dobj = repo.cacheHandler.open_database(db_un_fn)
 
             else:
+                repo._xml2sqlite_local = True
                 xml = repo_get_function()
                 xmldata = repo.repoXML.getData(mymdtype)
                 (ctype, csum) = xmldata.checksum
@@ -231,6 +234,7 @@ class YumRepository(Repository, config.RepoConf):
         Repository.__init__(self, repoid)
 
         self.repofile = None
+        self.mirrorurls = []
         self._urls = []
         self.enablegroups = 0
         self.groupsfilename = 'yumgroups.xml' # something some freaks might
@@ -384,7 +388,7 @@ class YumRepository(Repository, config.RepoConf):
             config.writeRawRepoFile(self,only=['enabled'])
         except IOError, e:
             if e.errno == errno.EACCES:
-                self.logger.warning(e)
+                logger.warning(e)
             else:
                 raise IOError, str(e)
 
@@ -395,7 +399,7 @@ class YumRepository(Repository, config.RepoConf):
             config.writeRawRepoFile(self,only=['enabled'])
         except IOError, e:
             if e.errno == errno.EACCES:
-                self.logger.warning(e)
+                logger.warning(e)
             else:
                 raise IOError, str(e)
 
@@ -473,7 +477,12 @@ class YumRepository(Repository, config.RepoConf):
                                     timeout=self.timeout,
                                     copy_local=self.copy_local,
                                     http_headers=headers,
-                                    reget='simple')
+                                    reget='simple',
+                                    ssl_verify_peer=self.sslverify,
+                                    ssl_verify_host=self.sslverify,
+                                    ssl_ca_cert=self.sslcacert,
+                                    ssl_cert=self.sslclientcert,
+                                    ssl_key=self.sslclientkey)
 
         self._grabfunc.opts.user_agent = default_grabber.opts.user_agent
 
@@ -495,6 +504,21 @@ class YumRepository(Repository, config.RepoConf):
     grabfunc = property(lambda self: self._getgrabfunc())
     grab = property(lambda self: self._getgrab())
 
+    def _dirSetupMkdir_p(self, dpath):
+        """make the necessary directory path, if possible, raise on failure"""
+        if os.path.exists(dpath) and os.path.isdir(dpath):
+            return
+
+        if self.cache:
+            raise Errors.RepoError, "Cannot access repository dir %s" % dpath
+
+        try:
+            os.makedirs(dpath, mode=0755)
+        except OSError, e:
+            msg = "%s: %s %s: %s" % ("Error making cache directory",
+                                     dpath, "error was", e)
+            raise Errors.RepoError, msg
+
     def dirSetup(self):
         """make the necessary dirs, if possible, raise on failure"""
 
@@ -510,19 +534,8 @@ class YumRepository(Repository, config.RepoConf):
         self.setAttribute('_dir_setup_metadata_cookie', cookie)
 
         for dir in [self.cachedir, self.pkgdir]:
-            if self.cache == 0:
-                if os.path.exists(dir) and os.path.isdir(dir):
-                    continue
-                else:
-                    try:
-                        os.makedirs(dir, mode=0755)
-                    except OSError, e:
-                        raise Errors.RepoError, \
-                            "Error making cache directory: %s error was: %s" % (dir, e)
-            else:
-                if not os.path.exists(dir):
-                    raise Errors.RepoError, \
-                        "Cannot access repository dir %s" % dir
+            self._dirSetupMkdir_p(dir)
+
         # if we're using a cachedir that's not the system one, copy over these
         # basic items from the system one
         self._preload_md_from_system_cache('repomd.xml')
@@ -530,17 +543,35 @@ class YumRepository(Repository, config.RepoConf):
         self._preload_md_from_system_cache('mirrorlist.txt')
         self._preload_md_from_system_cache('metalink.xml')
 
-    def _dirAttr(self, attr):
+    def _dirGetAttr(self, attr):
         """ Make the directory attributes call .dirSetup() if needed. """
         attr = '_dir_setup_' + attr
         if not hasattr(self, attr):
             self.dirSetup()
         return getattr(self, attr)
-    cachedir = property(lambda self: self._dirAttr('cachedir'))
-    pkgdir   = property(lambda self: self._dirAttr('pkgdir'))
-    hdrdir   = property(lambda self: self._dirAttr('hdrdir'))
-    gpgdir   = property(lambda self: self._dirAttr('gpgdir'))
-    metadata_cookie = property(lambda self: self._dirAttr('metadata_cookie'))
+    def _dirSetAttr(self, attr, val):
+        """ Make the directory attributes call .dirSetup() if needed. """
+        attr = '_dir_setup_' + attr
+        if not hasattr(self, attr):
+            self.dirSetup()
+
+        if attr == '_dir_setup_pkgdir':
+            if not hasattr(self, '_old_pkgdirs'):
+                self._old_pkgdirs = []
+            self._old_pkgdirs.append(getattr(self, attr))
+
+        ret = setattr(self, attr, val)
+        if attr in ('_dir_setup_pkgdir', ):
+            self._dirSetupMkdir_p(val)
+        return ret
+    cachedir = property(lambda self: self._dirGetAttr('cachedir'))
+    pkgdir   = property(lambda self: self._dirGetAttr('pkgdir'),
+                        lambda self, x: self._dirSetAttr('pkgdir', x))
+    hdrdir   = property(lambda self: self._dirGetAttr('hdrdir'),
+                        lambda self, x: self._dirSetAttr('hdrdir', x))
+    gpgdir   = property(lambda self: self._dirGetAttr('gpgdir'),
+                        lambda self, x: self._dirSetAttr('gpgdir', x))
+    metadata_cookie = property(lambda self: self._dirGetAttr('metadata_cookie'))
 
     def baseurlSetup(self):
         warnings.warn('baseurlSetup() will go away in a future version of Yum.\n',
@@ -602,6 +633,9 @@ class YumRepository(Repository, config.RepoConf):
         goodurls = []
         skipped = None
         for url in url_list:
+            # obvious bogons get ignored b/c, we could get more interesting checks but <shrug>
+            if url in ['', None]:
+                continue
             url = parser.varReplace(url, self.yumvar)
             if url[-1] != '/':
                 url= url + '/'
@@ -640,7 +674,12 @@ class YumRepository(Repository, config.RepoConf):
                                     retry = self.retries,
                                     throttle = self.throttle,
                                     progress_obj = self.callback,
-                                    proxies=self.proxy_dict)
+                                    proxies=self.proxy_dict,
+                                    ssl_verify_peer=self.sslverify,
+                                    ssl_verify_host=self.sslverify,
+                                    ssl_ca_cert=self.sslcacert,
+                                    ssl_cert=self.sslclientcert,
+                                    ssl_key=self.sslclientkey)
                     ug.opts.user_agent = default_grabber.opts.user_agent
                     result = ug.urlgrab(url, local, text=self.id + "/metalink")
 
@@ -651,7 +690,7 @@ class YumRepository(Repository, config.RepoConf):
                         raise Errors.RepoError, msg
                     #  Now, we have an old usable metalink, so we can't move to
                     # a newer repomd.xml ... or checksums won't match.
-                    print "Could not get metalink %s error was \n%s" %(url, e)
+                    print "Could not get metalink %s error was\n%s: %s" % (url, e.args[0], misc.to_unicode(e.args[1]))                    
                     self._metadataCurrent = True
 
             if not self._metadataCurrent:
@@ -675,7 +714,8 @@ class YumRepository(Repository, config.RepoConf):
                              fdel=lambda self: setattr(self, "_metalink", None))
 
     def _getFile(self, url=None, relative=None, local=None, start=None, end=None,
-            copy_local=None, checkfunc=None, text=None, reget='simple', cache=True):
+            copy_local=None, checkfunc=None, text=None, reget='simple', 
+            cache=True, size=None):
         """retrieve file from the mirrorgroup for the repo
            relative to local, optionally get range from
            start to end, also optionally retrieve from a specific baseurl"""
@@ -743,6 +783,12 @@ class YumRepository(Repository, config.RepoConf):
                             timeout=self.timeout,
                             checkfunc=checkfunc,
                             http_headers=headers,
+                            ssl_verify_peer=self.sslverify,
+                            ssl_verify_host=self.sslverify,
+                            ssl_ca_cert=self.sslcacert,
+                            ssl_cert=self.sslclientcert,
+                            ssl_key=self.sslclientkey,
+                            size=size
                             )
 
             ug.opts.user_agent = default_grabber.opts.user_agent
@@ -774,6 +820,7 @@ class YumRepository(Repository, config.RepoConf):
                                            reget = reget,
                                            checkfunc=checkfunc,
                                            http_headers=headers,
+                                           size=size
                                            )
             except URLGrabError, e:
                 errstr = "failure: %s from %s: %s" % (relative, self.id, e)
@@ -785,17 +832,23 @@ class YumRepository(Repository, config.RepoConf):
         return result
     __get = _getFile
 
-    def getPackage(self, package, checkfunc = None, text = None, cache = True):
+    def getPackage(self, package, checkfunc=None, text=None, cache=True):
         remote = package.relativepath
         local = package.localPkg()
         basepath = package.basepath
 
+        if self._preload_pkg_from_system_cache(package):
+            if package.verifyLocalPkg():
+                return local
+            misc.unlink_f(local)
+
         return self._getFile(url=basepath,
                         relative=remote,
                         local=local,
                         checkfunc=checkfunc,
                         text=text,
-                        cache=cache
+                        cache=cache,
+                        size=package.size,
                         )
 
     def getHeader(self, package, checkfunc = None, reget = 'simple',
@@ -805,6 +858,7 @@ class YumRepository(Repository, config.RepoConf):
         local =  package.localHdr()
         start = package.hdrstart
         end = package.hdrend
+        size = end-start
         basepath = package.basepath
         # yes, I know, don't ask
         if not os.path.exists(self.hdrdir):
@@ -812,7 +866,7 @@ class YumRepository(Repository, config.RepoConf):
 
         return self._getFile(url=basepath, relative=remote, local=local, start=start,
                         reget=None, end=end, checkfunc=checkfunc, copy_local=1,
-                        cache=cache,
+                        cache=cache, size=size,
                         )
 
     def metadataCurrent(self):
@@ -820,18 +874,28 @@ class YumRepository(Repository, config.RepoConf):
         age of the cookie is less than metadata_expire time then return true
         else return False. This result is cached, so that metalink/repomd.xml
         are synchronized."""
-        if self._metadataCurrent is None:
-            mlfn = self.cachedir + '/' + 'metalink.xml'
-            if self.metalink and not os.path.exists(mlfn):
-                self._metadataCurrent = False
-        if self._metadataCurrent is None:
-            repomdfn = self.cachedir + '/' + 'repomd.xml'
-            if not os.path.exists(repomdfn):
-                self._metadataCurrent = False
-        if self._metadataCurrent is None:
-            self._metadataCurrent = self.withinCacheAge(self.metadata_cookie,
-                                                        self.metadata_expire)
-        return self._metadataCurrent
+        if self._metadataCurrent is not None:
+            return self._metadataCurrent
+
+        mC_def = self.withinCacheAge(self.metadata_cookie, self.metadata_expire)
+        if not mC_def: # Normal path...
+            self._metadataCurrent = mC_def
+            return mC_def
+
+        # Edge cases, both repomd.xml and metalink (if used). Must exist.
+        repomdfn = self.cachedir + '/' + 'repomd.xml'
+        if not os.path.exists(repomdfn):
+            self._metadataCurrent = False
+            return False
+
+        self._hack_mirrorlist_for_anaconda()
+        mlfn = self.cachedir + '/' + 'metalink.xml'
+        if self.metalink and not os.path.exists(mlfn):
+            self._metadataCurrent = False
+            return False
+
+        self._metadataCurrent = True
+        return True
 
     #  The metalink _shouldn't_ be newer than the repomd.xml or the checksums
     # will be off, but we only really care when we are downloading the
@@ -931,7 +995,8 @@ class YumRepository(Repository, config.RepoConf):
                                    text=text,
                                    reget=None,
                                    checkfunc=checkfunc,
-                                   cache=self.http_caching == 'all')
+                                   cache=self.http_caching == 'all',
+                                   size=102400) # setting max size as 100K
 
         except URLGrabError, e:
             misc.unlink_f(tfname)
@@ -977,6 +1042,8 @@ class YumRepository(Repository, config.RepoConf):
             old_local = local + '.old.tmp' # locked, so this is ok
             shutil.copy2(local, old_local)
             xml = self._parseRepoXML(old_local, True)
+            if xml is None:
+                return None
             self._oldRepoMDData = {'old_repo_XML' : xml, 'local' : local,
                                    'old_local' : old_local, 'new_MD_files' : []}
             return xml
@@ -1289,13 +1356,6 @@ class YumRepository(Repository, config.RepoConf):
         self._doneOldRepoXML()
         return True
 
-    def _instantLoadRepoXML(self, text=None):
-        """ Retrieve the new repomd.xml from the repository, then check it
-            and parse it. If it fails revert.
-            Mostly traditional behaviour. """
-        if self._commonLoadRepoXML(text):
-            self._commonRetrieveDataMD([])
-
     def _groupLoadRepoXML(self, text=None, mdtypes=None):
         """ Retrieve the new repomd.xml from the repository, then check it
             and parse it. If it fails we revert to the old version and pretend
@@ -1309,19 +1369,25 @@ class YumRepository(Repository, config.RepoConf):
 
     def _loadRepoXML(self, text=None):
         """retrieve/check/read in repomd.xml from the repository"""
+        md_groups = {'instant'       : [],
+                     'group:primary' : ['primary'],
+                     'group:small'   : ["primary", "updateinfo"],
+                     'group:main'    : ["primary", "group", "filelists",
+                                        "updateinfo", "prestodelta"]}
+        mdtypes = set()
+        if type(self.mdpolicy) in types.StringTypes:
+            mdtypes.update(md_groups.get(self.mdpolicy, [self.mdpolicy]))
+        else:
+            for mdpolicy in self.mdpolicy:
+                mdtypes.update(md_groups.get(mdpolicy, [mdpolicy]))
+
+        if not mdtypes or 'group:all' in mdtypes:
+            mdtypes = None
+        else:
+            mdtypes = sorted(list(mdtypes))
+
         try:
-            if self.mdpolicy in ["instant"]:
-                return self._instantLoadRepoXML(text)
-            if self.mdpolicy in ["group:all"]:
-                return self._groupLoadRepoXML(text)
-            if self.mdpolicy in ["group:main"]:
-                return self._groupLoadRepoXML(text, ["primary", "group",
-                                                     "filelists", "updateinfo",
-                                                     "prestodelta"])
-            if self.mdpolicy in ["group:small"]:
-                return self._groupLoadRepoXML(text, ["primary", "updateinfo"])
-            if self.mdpolicy in ["group:primary"]:
-                return self._groupLoadRepoXML(text, ["primary"])
+            return self._groupLoadRepoXML(text, mdtypes)
         except KeyboardInterrupt:
             self._revertOldRepoXML() # Undo metadata cookie?
             raise
@@ -1362,7 +1428,8 @@ class YumRepository(Repository, config.RepoConf):
                                        text='%s/signature' % self.id,
                                        reget=None,
                                        checkfunc=None,
-                                       cache=self.http_caching == 'all')
+                                       cache=self.http_caching == 'all',
+                                       size=102400)
             except URLGrabError, e:
                 raise URLGrabError(-1, 'Error finding signature for repomd.xml for %s: %s' % (self, e))
 
@@ -1476,9 +1543,14 @@ class YumRepository(Repository, config.RepoConf):
         try:
             checkfunc = (self.checkMD, (mdtype,), {})
             text = "%s/%s" % (self.id, mdtype)
-            local = self._getFile(relative=remote, local=local, copy_local=1,
-                             checkfunc=checkfunc, reget=None, text=text,
-                             cache=self.http_caching == 'all')
+            local = self._getFile(relative=remote,
+                                  local=local, 
+                                  copy_local=1,
+                                  checkfunc=checkfunc, 
+                                  reget=None, 
+                                  text=text,
+                                  cache=self.http_caching == 'all',
+                                  size=thisdata.size)
         except (Errors.NoMoreMirrorsRepoError, Errors.RepoError):
             if retrieve_can_fail:
                 return None
@@ -1548,8 +1620,8 @@ class YumRepository(Repository, config.RepoConf):
             for line in content:
                 if re.match('^\s*\#.*', line) or re.match('^\s*$', line):
                     continue
-                mirror = re.sub('\n$', '', line) # no more trailing \n's
-                (mirror, count) = re.subn('\$ARCH', '$BASEARCH', mirror)
+                mirror = line.rstrip() # no more trailing \n's
+                mirror = mirror.replace('$ARCH', '$BASEARCH')
                 returnlist.append(mirror)
 
         return (returnlist, content)
@@ -1578,7 +1650,7 @@ class YumRepository(Repository, config.RepoConf):
             try:
                 fo = urlgrabber.grabber.urlopen(url, proxies=self.proxy_dict)
             except urlgrabber.grabber.URLGrabError, e:
-                print "Could not retrieve mirrorlist %s error was\n%s" % (url, e)
+                print "Could not retrieve mirrorlist %s error was\n%s: %s" % (url, e.args[0], misc.to_unicode(e.args[1]))
                 fo = None
 
         (returnlist, content) = self._readMirrorList(fo, url)
@@ -1596,7 +1668,19 @@ class YumRepository(Repository, config.RepoConf):
 
         return returnlist
 
-    def _preload_file_from_system_cache(self, filename, subdir=''):
+    def _preload_file(self, fn, destfn):
+        """attempts to copy the file, if possible"""
+        # don't copy it if the copy in our users dir is newer or equal
+        if not os.path.exists(fn):
+            return False
+        if os.path.exists(destfn):
+            if os.stat(fn)[stat.ST_CTIME] <= os.stat(destfn)[stat.ST_CTIME]:
+                return False
+        shutil.copy2(fn, destfn)
+        return True
+
+    def _preload_file_from_system_cache(self, filename, subdir='',
+                                        destfn=None):
         """attempts to copy the file from the system-wide cache,
            if possible"""
         if not hasattr(self, 'old_base_cache_dir'):
@@ -1611,16 +1695,10 @@ class YumRepository(Repository, config.RepoConf):
             return False
 
         # Try to copy whatever file it is
-        fn = glob_repo_cache_dir + '/' + subdir + os.path.basename(filename)
-        destfn = self.cachedir   + '/' + subdir + os.path.basename(filename)
-        # don't copy it if the copy in our users dir is newer or equal
-        if not os.path.exists(fn):
-            return False
-        if os.path.exists(destfn):
-            if os.stat(fn)[stat.ST_CTIME] <= os.stat(destfn)[stat.ST_CTIME]:
-                return False
-        shutil.copy2(fn, destfn)
-        return True
+        fn = glob_repo_cache_dir   + '/' + subdir + os.path.basename(filename)
+        if destfn is None:
+            destfn = self.cachedir + '/' + subdir + os.path.basename(filename)
+        return self._preload_file(fn, destfn)
 
     def _preload_md_from_system_cache(self, filename):
         """attempts to copy the metadata file from the system-wide cache,
@@ -1630,7 +1708,87 @@ class YumRepository(Repository, config.RepoConf):
     def _preload_pkg_from_system_cache(self, pkg):
         """attempts to copy the package from the system-wide cache,
            if possible"""
-        return self._preload_file_from_system_cache(pkg.localPkg(),subdir='packages/')
+        pname  = os.path.basename(pkg.localPkg())
+        destfn = os.path.join(self.pkgdir, pname)
+        if self._preload_file_from_system_cache(pkg.localPkg(),
+                                                subdir='packages/',
+                                                destfn=destfn):
+            return True
+
+        if not hasattr(self, '_old_pkgdirs'):
+            return False
+        for opkgdir in self._old_pkgdirs:
+            if self._preload_file(os.path.join(opkgdir, pname), destfn):
+                return True
+        return False
+
+    def _verify_md(self):
+        problems = []
+        print 'verifying md'
+        try:
+            md_types = self.repoXML.fileTypes()
+        except Errors.RepoError, e:
+            prb = RepoVerifyProblem(1, "failed to load repomd.xml", str(e))
+            problems.append(prb)
+            return problems
+
+        for md_type in md_types:
+            print 'verifying %s' % md_type
+            try:
+                self.retrieveMD(md_type)
+            except Errors.RepoError, e:
+                msg = "%s metadata missing or does not match checksum" % md_type
+                prb = RepoVerifyProblem(2, msg, str(e))
+                problems.append(prb)
+
+        return problems
+
+    def _verify_comps(self):
+        print 'verifying comps'
+        problems = []
+        # grab the comps for this repo
+        # run the xmllint on it
+        # chuck it into a comps object
+        # make sure it parses
+
+        grpfile = self.getGroups()
+
+        # open it up as a file object so iterparse can cope with our gz file
+        if grpfile is not None and grpfile.endswith('.gz'):
+            grpfile = gzip.open(grpfile)
+        try:
+            c = comps.Comps()
+            c.add(grpfile)
+        except (Errors.GroupsError, Errors.CompsException), e:
+            msg = "comps file failed to add"
+            prb = RepoVerifyProblem(REPO_PROBLEM_COMPS, msg, str(e))
+            problems.add(prb)
+        else:
+            if c.compscount == 0:
+                msg = "no groups in comps"
+                prb = RepoVerifyProblem(REPO_PROBLEM_COMPS, msg, "")
+                problems.add(prb)
+
+        return problems
+
+    def _verify_packages(self):
+        return []
+
+    def verify(self, items=['repodata', 'comps']):
+        """download/verify the specified items
+           @items = ['repodata', 'comps'] can include: repodata, comps, packages
+        """
+        problems = []
+        if 'repodata' in items:
+            problems.extend(self._verify_md())
+        if 'comps' in items:        
+            if self.enablegroups:
+                problems.extend(self._verify_comps())
+        if 'packages' in items:
+            problems.extend(self._verify_packages())
+        # what else can we verify?
+
+        return problems
 
 
 def getMirrorList(mirrorlist, pdict = None):
@@ -1656,7 +1814,7 @@ def getMirrorList(mirrorlist, pdict = None):
     try:
         fo = urlresolver.urlopen(url, proxies=pdict)
     except urlgrabber.grabber.URLGrabError, e:
-        print "Could not retrieve mirrorlist %s error was\n%s" % (url, e)
+        print "Could not retrieve mirrorlist %s error was\n%s: %s" % (url, e.args[0], misc.to_unicode(e.args[1]))
         fo = None
 
     if fo is not None:
@@ -1664,9 +1822,17 @@ def getMirrorList(mirrorlist, pdict = None):
         for line in content:
             if re.match('^\s*\#.*', line) or re.match('^\s*$', line):
                 continue
-            mirror = re.sub('\n$', '', line) # no more trailing \n's
-            (mirror, count) = re.subn('\$ARCH', '$BASEARCH', mirror)
+            mirror = line.rstrip() # no more trailing \n's
+            mirror = mirror.replace('$ARCH', '$BASEARCH')
             returnlist.append(mirror)
 
     return returnlist
 
+class RepoVerifyProblem:
+    """ Holder for each "problem" we find with a repo.verify(). """
+    
+    def __init__(self, type, msg, details, fake=False):
+        self.type           = type
+        self.message        = msg
+        self.details        = details
+        self.fake           = fake
