@@ -31,7 +31,7 @@ from yum import _
 from yum.i18n import to_unicode
 import yum.misc
 import cli
-
+from utils import suppress_keyboard_interrupt_message
 
 def main(args):
     """This does all the real work"""
@@ -46,6 +46,8 @@ def main(args):
     def exIOError(e):
         if e.errno == 32:
             logger.critical(_('\n\nExiting on Broken Pipe'))
+        else:
+            logger.critical(_('\n\n%s') % str(e))
         if unlock(): return 200
         return 1
 
@@ -305,12 +307,18 @@ def user_main(args, exit_code=False):
             errcode = cprof(main, args)
         if os.environ['YUM_PROF'] == 'hotshot':
             errcode = hotshot(main, args)
+    if 'YUM_PDB' in os.environ:
+        import pdb
+        pdb.run(main(args))
+
     if errcode is None:
         errcode = main(args)
     if exit_code:
         sys.exit(errcode)
     return errcode
 
+suppress_keyboard_interrupt_message()
+
 if __name__ == "__main__":
     try:
         user_main(sys.argv[1:], exit_code=True)
